# -*- coding: utf-8 -*-
import sqlite3
import os
from urllib import urlretrieve


import subprocess
from common import messager, settings, util, xdg
from music.tags import Tags
import threading
import time
from datetime import datetime
import elements
import unicodedata
import logging
from pylast import LastFMNetwork, NetworkError, WSError, md5


logger = logging.getLogger(__name__)


class DB():
	def __init__(self):
		db_path = os.path.join(xdg.get_data_home(), 'data.db')
		if os.path.exists(db_path):
			self.conn = sqlite3.connect(db_path)
			self.conn.row_factory = sqlite3.Row
			self.c = self.conn.cursor()
		#else:
			#self.conn = sqlite3.connect('data.db')
			#self.conn.row_factory = sqlite3.Row
			#self.c = self.conn.cursor()
			#self.createTables()
	
	def getRow(self, query, t=None):
		if(t != None):
			self.c.execute(query, t)
		else:
			self.c.execute(query)
		row = self.c.fetchone()
		return row
		
	def getRows(self, query, t=None):
		if(t != None):
			self.c.execute(query, t)
		else:
			self.c.execute(query)
		rows = self.c.fetchall()
		return rows
		
		
	def execute(self, query, t=None):
		'''
			Shortcut method used for saving queries (INSERT / UPDATE)
		'''
		if(t != None):
			self.c.execute(query, t)
		else:
			self.c.execute(query)
		logger.debug(query)
		self.conn.commit()
	
	def execute_with_filters(self, query, module, params=[]):
		'''
			Execute query adding module filters from settings in WHERE clause
		'''
		filters = settings.get_option(module + '/filters', {})
			
		for key in filters.iterkeys():
			if filters[key]['enabled']:
				try:
					where += ' OR '
				except NameError:
					where = 'WHERE '
				for t in filters[key]['criterions']:
					try:
						filter += filters[key]['link'] + t[0] + t[1] + '? '
					except NameError:
						filter = t[0] + t[1] + '? '
					params.append(t[2])
					
				where += '(' + filter + ')'
				del filter
		
		try:
			query += where
		except NameError:
			pass
		
		self.c.execute(query, params)

		
	
		
	
	def getTracksFromIDs(self, IDs):
		'''
			Return a list of track matching the ID List. Used for playlist loading and queues sessions loading
		'''
		# IN (%s)''' % ','.join('?'*len(IDs)), (IDs,))
		self.c.execute('SELECT * FROM tracks WHERE track_ID IN (%s)' % ("?," * len(IDs))[:-1], IDs)
		
		
		tracksData = []
		for row in self.c:
			tracksData.append((row[0], row[1], row[2], row[3], row[4], row[6], row[7], row[8]))
		
		# SQLite ordered tracks in IDs order, not what we want
		SQLOrder = sorted(IDs)
		
		posInSQLOrder = dict()
		for i in range(len(SQLOrder)):
			posInSQLOrder[SQLOrder[i]] = i

		tracks = []
		for ID in IDs:
			tracks.append(elements.Track(tracksData[posInSQLOrder[ID]]))
		return tracks
		
		
	def getTracks(self, dic):
		'''
			Return a list of tracks matching dict, eg : {"Artist":"ZZ Top", "Album":"Mescalero"}
		'''
		tracks = []
		tracksData = self.get_tracks_data(dic)
		for dataTuple in tracksData:
			tracks.append(elements.Track(dataTuple))
		return tracks
		
		
	def getTracksOld(self, dic, filters={}):
		'''
			DEPRECATED This method doesn't use get_tracks_data and thus involves a lot more SQL Queries than getTracks
		'''
		params = []
		for key in dic.iterkeys():
			try:
				query += ' AND ' + key + ' = ?'
			except NameError:
				query = 'SELECT track_ID FROM tracks WHERE ' + key + ' = ?'
			params.append(unicode(dic[key]))

			for t in filters['criterions']:
				query += ' ' + filters['link'] + t[0] + t[1] + '? '
				params.append(t[2])
		
		d = self.conn.cursor()
		d.execute(query, params)
		tracks = []
		for row in d:
			tracks.append(elements.Track(row[0])) # <---- Involve another SQL Query
		return tracks
		
		
	def get_tracks_data(self, data):
		'''
			@return : list (python array) of tracks data matching dict(s) contained in param data
			@param data : might be one dictionnary (AND SQL Clause), or a list of dictionnary (OR SQL Clause)
		'''
		t = []
		if type(data).__name__=='dict':
			for key in data.iterkeys():
				try:
					query += ' AND ' + key + ' = ?'
				except NameError:
					query = 'SELECT * FROM tracks WHERE ' + key + ' = ?'
				t.append(unicode(data[key]))
		else:
			for dic in data:
				try:
					query += ' OR ('
				except NameError:
					query = 'SELECT * FROM tracks WHERE ('
				for key in dic.iterkeys():
					try:
						part += ' AND ' + key + ' = ?'
					except NameError:
						part = key + ' = ?'
					t.append(unicode(dic[key]))
					
				query += part + ')'
				del part
				
				
		self.c.execute(query, t)
		table = []
		for row in self.c:
			table.append((row[0], row[1], row[2], row[3], row[4], row[6], row[7], row[8]))

		return table
		
	@staticmethod
	@util.threaded
	def initNetwork():
		'''
			Connect Bullseye to Last.fm account stored in settings
		'''
		DB.network_is_connected = threading.Event()
		DB.network_cache = []
		try:
			f = open(os.path.join(xdg.get_data_home(), 'network_cache.txt'), 'r')
			queue = f.readlines()
			f.close()
			for e in queue:
				DB.network_cache.append(eval(e))
		except IOError:
			logger.debug("No network cache file")
		try:
			API_KEY = "04537e40b5501f85610cf4e4bbf1d97a" # this is a sample key
			API_SECRET = "b36376228a6e72314ffd503b8e3c9f5e"

			# In order to perform a write operation you need to authenticate yourself
			username = settings.get_option('music/audioscrobbler_login', '')
			password_hash = md5(settings.get_option('music/audioscrobbler_password', ''))

			DB.network = LastFMNetwork(api_key = API_KEY, api_secret = 
			API_SECRET, username = username, password_hash = password_hash)
			logger.debug('Connection successfuly established with Last.fm')
			DB.network_is_connected.set()
		except NetworkError:
			logger.debug('Connection to Last.fm failed')
		
		
	@staticmethod
	def saveCache():
		'''
			Save Last.fm pending scrobbles in file
		'''
		try:
			f = open(os.path.join(xdg.get_data_home(), 'network_cache.txt'), 'w')
			for e in DB.network_cache:
				f.write(str(e) + "\n")
			f.close()
		except:
			logger.debug('Error saving scrobbling cache')


class MainDB(DB):
	"""
		A class used for utility purpose and GUI access to database
		There should be only one instance of this class
		TODO Re-arrange folders analysis
		TODO standardize methods to get elements with filters (cf retrieve, load, etc)
		TODO Make this class a utility class (static)
	"""
	def __init__(self):
		db_path = os.path.join(xdg.get_data_home(), 'data.db')
		if os.path.exists(db_path):
			self.conn = sqlite3.connect(db_path)
			self.conn.row_factory = sqlite3.Row
			self.c = self.conn.cursor()
		else:
			self.conn = sqlite3.connect(db_path)
			self.conn.row_factory = sqlite3.Row
			self.c = self.conn.cursor()
			self.createTables()
		
		
			
			
		
		DB.initNetwork()

		
		logger.debug('DB initialized')
		



	
	
	@util.threaded
	def checkForNewFiles(self, progressWidget, folders=None):
		'''
			Main method used to fill database
			@param folders : list of (str path, bool dig) 
			@param progressWidget : used for progress feedback
			TODO walk + list (see Bluemindo)
			TODO use trigger to check existence - not needed with current process
			TODO PERF trick : try to check existence with a trigger but without retrieving tags
			TODO prompt errors
		'''
		
		if folders is None:
			folders = settings.get_option('music/folders', [])
	
	
		conn = sqlite3.connect(os.path.join(xdg.get_data_home(), 'data.db'))
		conn.row_factory = sqlite3.Row
		c = conn.cursor()

		recovered_paths = []
		error_paths = []
		
		indexedExtensions = {}
		indexedExtensions['music'] = ('.ogg', '.mp3')
		indexedExtensions['pictures'] = (".jpg", '.gif', ".png", ".bmp")
		indexedExtensions['videos'] = (".flv", ".mkv", ".avi")
		for ucModule in settings.get_option('general/custom_modules', []):
			indexedExtensions[ucModule] = settings.get_option(ucModule + '/indexed_extensions')
		
		def addFilesIfNeeded(files):
			'''
				Add new files to DB
				
				@param files : tuple list(type, folder, filename, extension)
				@param progressWidget : provide feedback to user
			'''
			
			
			# --- First we make sure to only keep paths that are truly new --- 
			registered_paths = []
			new_files = {}
			for module in indexedExtensions.iterkeys():
				new_files[module] = []
				if module == 'music':
					c.execute('SELECT path FROM tracks')
					for row in c:
						registered_paths.append(row[0])
				else:
					c.execute('SELECT folder, filename, folder FROM ' + module)
					for row in c:
						registered_paths.append(row[0] + os.sep + row[1])
				
			new_paths = list(set(recovered_paths) - set(registered_paths))
			
			
			# --- Then module by module we retrieve data for these files ---
			for module in indexedExtensions.iterkeys():
				size = float(len(files[module]))
				progressWidget.setFraction(0)
				i = 0
				if module == 'music':
					for track in files['music']:
						path = track[0] + os.sep + track[1]
						if(path in new_paths):
							new_files['music'].append(get_track_data(path))
						i += 1
						progressWidget.setFraction((float(i) / size))
				else: # UC Module
					for element in files[module]:
						path = element[0] + os.sep + element[1]
						if(path in new_paths):
							new_files[module].append(get_UC_element_data(module, element[0], element[1]))
						i += 1
						progressWidget.setFraction((float(i) / size))
			
			

			# --- Finally, we insert retrieved data into database ---
			for module in new_files.iterkeys():
				if(module == 'music'):
					conn.executemany('INSERT INTO tracks (path, title, album, artist, genre, length, rating, playcount, year, performer) VALUES (?,?,?,?,?, ?, ?, ?, ?, ?)', new_files[module])
				else:
					conn.executemany('INSERT INTO ' + module + ' (folder, filename, rating, categorie_ID, univers_ID, size) VALUES (?, ?, ?, ?, ?, ?)', new_files[module])
			
			progressWidget.setFraction(0)
			conn.commit()
				
		def get_UC_element_data(module, folder, filename):
			t = (unicode(folder), unicode(filename), 0, 1, 1, os.path.getsize(os.path.join(folder, filename)))
			return t
			
		
		def get_track_data(path):
			tags = Tags.fromPath(path)
			path = unicode(path)
			t = (path, tags['title'], tags['album'], tags['artist'], tags['genre'], tags['length'], 0, 0, tags['year'], tags['performer'])
			return t
			
			
		def scanFolder(folder, dig=False, files={}):
			'''
				Scan a folder (and recusively sub-folders if dig) to locate potential files
				@param folder : path of folder to be scanned
				@param files : a dict containing lists of tuples containg data on new files
			'''
			

			def checkFileInterest(folder, filename, extension):
				for module in indexedExtensions.iterkeys():
					if extension in indexedExtensions[module]:
						try:
							files[module].append((folder, filename))
							recovered_paths.append(unicode(folder + os.sep + filename))
						except UnicodeDecodeError:
							error_paths.append(folder + os.sep + filename)
						break

			progressWidget.pulse()
			for f in os.listdir(folder):
				if f[0] != ".":
					if os.path.isfile(os.path.join(folder, f)):
						(shortname, extension) = os.path.splitext(f)
						checkFileInterest(folder, f, extension)
					else:
						if(dig):
							scanFolder(folder + os.sep + f, dig, files)
			return (files)
		
		for folder in folders:
			files = {}
			for module in indexedExtensions.iterkeys():
				files[module] = []
			new_files = scanFolder(folder[0], folder[1], files)
			addFilesIfNeeded(new_files)
			
		for path in error_paths:
			logging.debug('Incorrect path : ' + path)
			
		progressWidget.emitDone()
			
		
	def createTables(self):
		'''
			Create database structure
		'''
		# ***** Music *****
		self.c.execute('''CREATE TABLE tracks
		(track_ID INTEGER PRIMARY KEY AUTOINCREMENT, path text, title text, album text,
		artist text, genre text, length int, playcount int, rating int, year text, num int, performer text, status text DEFAULT 'a')''')
		
		
		self.c.execute('''CREATE TRIGGER insert_track BEFORE INSERT ON tracks
			FOR EACH ROW BEGIN
				SELECT  RAISE(IGNORE)
				WHERE (SELECT track_ID from tracks WHERE path = NEW.path) IS NOT NULL;
			END;''')
			
			
		self.c.execute('''CREATE TABLE listened (track_ID INTEGER, time INTEGER, PRIMARY KEY(track_ID, time));''')
		
		
		for ucModule in ('video', 'picture'):
			self.createUCTablesFor(ucModule)

		#self.c.execute('''CREATE TABLE link_IC
		#(categorie_ID int, image_ID int, PRIMARY KEY(categorie_ID, image_ID))''')
		
		
	def createUCTablesFor(self, module):
		queries = []
		queries.append( 'CREATE TABLE univers_' + module + 's (univers_ID INTEGER PRIMARY KEY AUTOINCREMENT, univers_L text, parent_ID INTEGER DEFAULT 0, thumbnail_ID INTEGER DEFAULT 0)')
		
		queries.append('CREATE TABLE categorie_' + module + 's (categorie_ID INTEGER PRIMARY KEY AUTOINCREMENT, categorie_L text, parent_ID INTEGER DEFAULT 0, thumbnail_ID INTEGER DEFAULT 0)')
		
		queries.append( 'INSERT INTO categorie_' + module + 's VALUES(1, "Divers",0 , 0)')
		queries.append ( 'INSERT INTO univers_' + module + 's VALUES(1, "Divers", 0, 0)')
		
		query = 'CREATE TABLE ' + module + 's ('+ module + '_ID INTEGER PRIMARY KEY AUTOINCREMENT, folder text, filename text, rating int, categorie_ID int, univers_ID int, size int, status text DEFAULT "a", '
		query += 'FOREIGN KEY(categorie_ID) REFERENCES categorie_' + module + 's(categorie_ID),'
		query += 'FOREIGN KEY(univers_ID) REFERENCES univers_' + module + 's(univers_ID))'
		queries.append(query)
		for query in queries:
			self.c.execute(query)
		



		

	

		
		
	def getDynamicListTracks(self, filter):
		'''
			Create a SQL Query to select tracks from filter data
		'''
		print filter
		params = []
		for t in filter['criterions']:
			try:
				query += ' ' + filter['link'] + t[0] + t[1] + '? '
			except NameError:
				query = 'SELECT * FROM tracks WHERE ' + t[0] + t[1] + '? '
			params.append(t[2])
		
		
		#COLLATE NOCASE'
		
		if(filter['random']):
			query += " ORDER BY RANDOM()"

		logger.debug(query)
		logger.debug(params)
		self.c.execute(query, params)
		table = []
		for row in self.c:
			table.append(elements.Track((row[0], row[1], row[2], row[3], row[4], row[6], row[7], row[8])))
		return table
		

		
	@staticmethod
	def formatLength(length):
		minutes = 0
		while length > 59:
			length -= 60
			minutes += 1
		if length < 10:
			length = "0" + str(length)
		else:
			length = str(length)
		length = str(minutes) + ":" + length
		
		return length
		
	
	def loadTracks(self, mode):
		'''
			Load data of every track organized in a structure defined by mode (eg of mode ('artist', 'album', 'title'))
			Used by LibraryPanel
		'''
		if settings.get_option('music/use_performer', False):
			artist_query = 'IFNULL(performer, artist) AS artist'
		else:
			artist_query = 'artist'

		query = 'SELECT track_ID, path, title, album, ' + artist_query + ', genre, length, playcount, rating, year, num FROM tracks '
		
		filters = settings.get_option('music/filters', {})
		params = []

		for key in filters.iterkeys():
					if filters[key]['enabled']:
						try:
							where += ' OR '
						except NameError:
							where = 'WHERE '
						for t in filters[key]['criterions']:
							try:
								filter += filters[key]['link'] + t[0] + t[1] + '? '
							except NameError:
								filter = t[0] + t[1] + '? '
							params.append(t[2])
							
						where += '(' + filter + ')'
						del filter

		try:
			query += where
		except NameError:
			pass
		query += ' ORDER BY ';
		columns = eval(mode)
		for e in columns:
			query += e + ', '
		print(query)
		self.c.execute(query[:-2], params)
		return self.c.fetchall()
		
		

	
	def quit(self):
		f = open(os.path.join(xdg.get_data_home(), 'network_cache.txt'), 'w')
		for e in DB.network_cache:
			f.write(str(e) + "\n")
		f.close()
	
	
	@util.threaded
	def reloadCovers(self, progressWidget):
		'''
			Function that download artists & albums covers from Last.fm
		'''
		forceReload = False
		artist_dir = xdg.get_thumbnail_dir('artist')
		album_dir = xdg.get_thumbnail_dir('album')
		
		
		if(DB.network_is_connected.isSet()):
			bdd = DB()
			
			bdd.execute_with_filters('SELECT DISTINCT artist, album FROM tracks ', 'music')
			
			rows = bdd.c.fetchall()
			length = float(len(rows))
			
			i = 0
			progressWidget.setFraction(0)
			while(i < len(rows)):
				artist_name = rows[i][0]
				dest_path = os.path.join(artist_dir + '/medium', artist_name.replace ('/', ' ')) # + '.jpg')
				if(forceReload or not os.path.isfile(dest_path)):
					artist = DB.network.get_artist(artist_name)
					cover = artist.get_cover_image(1)
					os.path
					
					if(cover is not None):
						#extension = os.path.splitext(f)[1]
						urlretrieve(cover, dest_path)
					else:
						logger.debug('No cover url for artist ' + artist_name)
				
				while(i < len(rows) and artist_name == rows[i][0]):
					album_name = rows[i][1]
					dest_path = os.path.join(album_dir + '/medium', album_name.replace ('/', ' ')) # + '.jpg')
					if(forceReload or not os.path.isfile(dest_path)):
						try:
							album = DB.network.get_album(artist_name, album_name)
							cover = album.get_cover_image(1)
						except WSError:
							logger.debug('Error : No album ' + album_name)
						
						if(cover is not None):
							urlretrieve(cover, dest_path)
						else:
							logger.debug('No cover url for album ' + album_name)
					i += 1
					progressWidget.setFraction(i / length)

		else:
			logger.debug("Can't retrieve covers : Network not connected")
		progressWidget.emitDone()
	
			
			


	
	def retrieveFromLastFM(self, progressWidget):
		"""
			Retrieve playcounts from Last.fm account
			TODO? Other account support?
			TODO? report
			TODO? album an option or rated ***
		"""
		def process():
			bdd = DB()
			account = settings.get_option('music/audioscrobbler_login', None)
			if account is not None:
				
				library = DB.network.get_user("Maitre_Piccolo").get_library()
				
				bdd.execute_with_filters('SELECT DISTINCT artist FROM tracks ', 'music')
				artists = bdd.c.fetchall()
				
				bdd.execute_with_filters('SELECT COUNT(DISTINCT artist, album) FROM tracks ', 'music')
				length = float(len(bdd.c.fetchone()[0]))
				nb_errors = 0
				i = 0
				progressWidget.setFraction(i)
				
				for artist in artists:
					try:
						albums = library.get_albums(artist[0])
						#if tracks empty check with only artist
						d = bdd.conn.cursor()
						for album in albums:
							album_title = album.item.get_title()
							
							tracks = library.get_tracks(artist[0], album_title)
							for track in tracks:
								title = track.item.get_title()
								bdd.c.execute('SELECT COUNT(title) FROM tracks WHERE artist = ? AND album != ? COLLATE NOCASE AND title = ? COLLATE NOCASE', (artist[0], album_title, title))
								#Ex : ZZ Top -> Eliminator -> Legs [Last.fm count 50][otherAlbumCount 0] and ZZ Top -> Greatest Hits -> Legs [Last.fm count 2][otherAlbumCount 1]
								# We don't want the second to erase the first BUT if we only have Greatest Hits we want the big score
								otherAlbumCount = bdd.c.fetchone()[0]
								
								if( otherAlbumCount > 0): #If song is in another album, we have to set the album
									t = (track.playcount, artist[0], album_title, title)
									d.execute('UPDATE tracks SET compteur = ? WHERE artist = ? AND album = ? COLLATE NOCASE AND title = ? COLLATE NOCASE', t)
								else: # There is no song with anoter album so we apply on all albums (useful if we have ONLY "Greatest Hits" album like, etc...)
									t = (track.playcount, artist[0], title)
									d.execute('UPDATE tracks SET compteur = ? WHERE artist = ? AND title = ? COLLATE NOCASE', t)
									
							i += 1
							progressWidget.setFraction(float(i) / length)
					except WSError:
						logger.debug('Error with artist ' + artist[0] + ' and album ' + album_title)
						nb_errors += 1
						progressWidget.setText(str(nb_errors) + ' ' + _('errors'))
				
				bdd.conn.commit()
				progressWidget.setText( _('Sync ended with') + ' ' + str(nb_errors) + ' ' + _('errors'))
				progressWidget.emitDone()
				
		task = threading.Thread(target=process)
		task.start()
		
		
	def retrieveFromSave(self, file_path, dic):
		'''
			Retrieve rating from another Bullseye base
		'''
		query = ''
		params = []
		for param in dic['criterions']:
				query += ' AND ' + param[0] + param[1] + ' ? '
				params.append(param[2])

		conn = sqlite3.connect(file_path)
		conn.row_factory = sqlite3.Row
		c = conn.cursor()
		c.execute('SELECT artist, album, title, rating, playcount FROM tracks')
		bigParamsArray = []
		for row in c:
			t = [row[3], row[0], row[1], row[2]]
			t.extend(params)
			bigParamsArray.append(t)
			#self.c.execute('UPDATE tracks SET note = ? WHERE artist = ? AND album = ? AND title = ?' + query, t)
		self.c.executemany('UPDATE tracks SET rating = ? WHERE artist = ? AND album = ? AND title = ?' + query, bigParamsArray)
		self.conn.commit()
		conn.close()