# -*- coding: utf-8 -*-
import gobject
import gtk
import os

from common import messager, xdg

from data import elements

from gui import modales
from gui.util import etoiles

class CreatePlaylistMenu(gtk.Menu):
	def __init__(self):
		gtk.Menu.__init__(self)
		
		i = gtk.ImageMenuItem(_("Add a dynamic playlist"))
		image = gtk.image_new_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU)
                i.set_image(image)
                i.connect('activate', self.addDynamicPlaylist)
		self.append(i)
		
		i = gtk.ImageMenuItem(_("Add a static playlist"))
		image = gtk.image_new_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU)
		i.set_image(image)
		i.connect('activate', self.addPlaylist)
		self.append(i)
		
		self.show_all()
		
	def addDynamicPlaylist(self, button):
		d = modales.DynamicPlaylistCreator()
		
	def addPlaylist(self, button):
		DN = gtk.Dialog()
		Entry = gtk.Entry()
		hbox = gtk.HBox()
		hbox.pack_start(gtk.Label(_("Name") + " : "), False, 5, 5)
		hbox.pack_end(Entry)
		DN.vbox.add(hbox)
		
		DN.add_button('OK', 1)
		DN.add_button(_("Cancel"), 0)
		DN.show_all()
		action = DN.run()
		name = Entry.get_text()
		DN.destroy()
		if(action == 1):
			f = open(os.path.join(xdg.get_data_home(), 'playlists' + os.sep) + name,'w')
			f.close()
			messager.emit('playlistAdded', self, ['static', name])


class MenuSpecialItem(gtk.Menu):
	"""
		menu popped on standard element (the ones with universe & categories stuff) right click
	"""
	def __init__(self, module, tuple_ID):
		self.module = module
		gtk.Menu.__init__(self)
		i = gtk.MenuItem(str(tuple_ID))
		file = elements.SpecialElement(tuple_ID, module)

		self.element = file
		rating_item = etoiles.RatingMenuItem(file.rating, False)
		rating_item.connect('rating-changed', self.onRatingChanged)
		self.append(rating_item)
		
		
		self.append(i)
		i = gtk.MenuItem(self.element.path)
		self.append(i)
		
		i = gtk.MenuItem(str(self.element.size))
		self.append(i)
		
		i = gtk.ImageMenuItem(_("Set thumbnail as category thumbnail"))
		image = gtk.image_new_from_stock(gtk.STOCK_HOME, gtk.ICON_SIZE_MENU)
		i.set_image(image)
		i.connect('activate', self.on_set_container_thumbnail, 'categorie')
		self.append(i)
		
		i = gtk.ImageMenuItem(_("Set thumbnail as universe thumbnail"))
		image = gtk.image_new_from_stock(gtk.STOCK_HOME, gtk.ICON_SIZE_MENU)
		i.set_image(image)
		i.connect('activate', self.on_set_container_thumbnail, 'univers')
		self.append(i)
		
		i = gtk.ImageMenuItem(_("Move to category folder"))
		image = gtk.image_new_from_stock(gtk.STOCK_HOME, gtk.ICON_SIZE_MENU)
		i.set_image(image)
		i.connect('activate', self.move)
		self.append(i)
		
		self.show_all()
		
	def onRatingChanged(self, widget, rating):
		self.element.setRating(rating)
		
	def on_set_container_thumbnail(self, menuitem, container_type):
		if(container_type == 'categorie'):
			container_ID = self.element.c_ID
		else:
			container_ID = self.element.u_ID
		container = elements.Container(container_ID, container_type, self.module)
		container.set_thumbnail_ID(self.element.ID)
	
	def move(self, *args):
		# potentially remove os from import
		old_path = self.element.path
		new_path = '/home/piccolo/Images/' + str(self.element.c_ID) + '/' + self.element.file
		print("TODO")
		#self.element.set_path(
		#os.renames(old_path, new_path)
		


class MenuCU(gtk.Menu):
	"""
		menu popped on UC container (category, universe) right click
	"""
	
	__gsignals__ = { "container-added": (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)) }
	
	def __init__(self, container_type, module, id):
		self.module = module
		if(container_type == 'u'):
			container_type = 'univers'
		elif(container_type == 'c'):
			container_type = 'categorie'
		if(container_type != 'unknown'):
			self.container = elements.Container(id, container_type, self.module)
		gtk.Menu.__init__(self)
		
		i = gtk.ImageMenuItem(_("Add an universe"))
		image = gtk.image_new_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU)
		i.set_image(image)
		i.connect_object('activate', self.addChildContainer, "universe", id)
		self.append(i)
		i = gtk.ImageMenuItem(_("Add a category"))
		image = gtk.image_new_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU)
		i.set_image(image)
		i.connect_object('activate', self.addChildContainer, "category", id)
		self.append(i)
		
		i = gtk.ImageMenuItem(_("Remove"))
		image = gtk.image_new_from_stock(gtk.STOCK_DELETE, gtk.ICON_SIZE_MENU)
		i.set_image(image)
		i.connect('activate', self.delete)
		self.append(i)
		
		self.show_all()
		
	def addChildContainer(self, type, parent_id):
		dialog = gtk.Dialog(buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
		box = dialog.get_content_area()
		hbox = gtk.HBox()
		hbox.pack_start(gtk.Label(_("Name") + " : "), False, 5, 5)
		entry = gtk.Entry()
		hbox.pack_start(entry)
		box.pack_start(hbox)
		box.show_all()
		response = dialog.run()
		newName = entry.get_text()
		dialog.destroy()
		if(response == gtk.RESPONSE_ACCEPT): # Validate
			if(type == "universe"):
				newContainer = elements.Container.create('univers', self.module, newName, parent_id)
			else:
				newContainer = elements.Container.create('categorie', self.module, newName, parent_id)
			self.emit('container-added', newContainer)
		elif(response == gtk.RESPONSE_REJECT): # Cancel
			return False
	
	def delete(self, button):
		dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _('Are you sure you want to delete container ' + self.container.label + '?'))
		resp = dialog.run()
		if resp == gtk.RESPONSE_YES:
			self.container.delete()
		dialog.destroy()
	
		
class PlaylistMenu(gtk.Menu):
	def __init__(self, playlistBrowserInstance, iter):
		gtk.Menu.__init__(self)
		i = gtk.ImageMenuItem(gtk.STOCK_DELETE)
		i.connect_object('activate', playlistBrowserInstance.deletePlaylist, iter)
		self.append(i)
		i = gtk.ImageMenuItem(gtk.STOCK_EDIT)
		i.connect_object('activate', playlistBrowserInstance.edit, iter)
		self.append(i)
		self.show_all()

		
class TrackMenu(gtk.Menu):
	def __init__(self, ID):
		gtk.Menu.__init__(self)
		track = elements.Track(ID)
		i = gtk.MenuItem(_("Title") + " : " + track.title)
		self.append(i)
		
		rating_item = etoiles.RatingMenuItem(track.rating, auto_update=False)
		rating_item.connect('rating-changed', track.change_rating)
		self.append(rating_item)
		
		i = gtk.ImageMenuItem(_("Edit tags"))
		i.set_image(gtk.image_new_from_stock(gtk.STOCK_EDIT, gtk.ICON_SIZE_MENU))
		i.connect('activate', self.editTrack, ID)
		self.append(i)

	def editTrack(self, button, track_ID):
		dialog = modales.TagsEditor(track_ID)
		
class TrackContainerMenu(gtk.Menu):
	def __init__(self, dic):
		gtk.Menu.__init__(self)
		#i = gtk.MenuItem(dic[len(dic)])
		#self.append(i)
		i = gtk.ImageMenuItem(_("Edit tags"))
		i.set_image(gtk.image_new_from_stock(gtk.STOCK_EDIT, gtk.ICON_SIZE_MENU))
		i.connect('activate', self.edit, dic)
		self.append(i)
		
		i = gtk.ImageMenuItem(_("Delete from DB"))
		i.set_image(gtk.image_new_from_stock(gtk.STOCK_DELETE, gtk.ICON_SIZE_MENU))
		i.connect('activate', self.delete, dic)
		self.append(i)


	def edit(self, button, dic):
		dialog = modales.TagsEditor(dic)
		
	def delete(self, button, dic):
		print('TODO DELETE')
		
class TrayMenu(gtk.Menu):
	def __init__(self, MainApp):
		gtk.Menu.__init__(self)
		i = gtk.ImageMenuItem(gtk.STOCK_QUIT)
		i.connect('activate', MainApp.on_window_destroy)
		self.append(i)
		self.show_all()
		
	def popup(self, status_icon, button, activate_time):
		gtk.Menu.popup(self, None, None, None, button, activate_time)
		
		
	