# -*- coding: utf-8 -*-
import os
import gtk, glib
import logging

from common import messager, settings, xdg
from data import elements

logger = logging.getLogger(__name__)


class Criterion(gtk.HBox):
	
	FIELDS = (('artist', _('Artist')), ('album', _('Album')), ('rating', _('Rating')), ('playcount', _('Playcount')), ('path', _('Path')))
	NUMERIC_OPERATORS = ((" = ", _("equals")), (" < ", _("is inferior to")), (" > ", _("is superior to")), (" <= ", _("is lower than")), (" >= ", _("is at least")))
	OPERATORS = ((" = ", _('is')), (" != ", _('is not'), ), (" LIKE ", _("like")), (" NOT LIKE ", _("not like")))
	
	FIELDMODEL = gtk.ListStore(str, str)
	for key, label in FIELDS:
		FIELDMODEL.append((key, label))
		
	OPERATORMODEL = gtk.ListStore(str, str)
	for key, label in OPERATORS:
		OPERATORMODEL.append((key, label))
		
	NUMERIC_OPERATORSMODEL = gtk.ListStore(str, str)
	for key, label in NUMERIC_OPERATORS:
		NUMERIC_OPERATORSMODEL.append((key, label))
	
	
	def __init__(self, config):
		'''
			config is a tuple containing a field, an operator and a value. eg ('artist', ' = ', 'AC/DC')
		'''
		gtk.HBox.__init__(self, spacing=12)
		self.set_border_width(2)
		
		self.fieldCB = gtk.ComboBox()
		self.fieldCB.set_model(self.FIELDMODEL)
		cell = gtk.CellRendererText()
		self.fieldCB.pack_start(cell)
		self.fieldCB.add_attribute(cell, "text", 1)
		
		
		pos = 0
		toMatch = config[0]
		while pos < len(self.FIELDS) and not self.FIELDS[pos][0] == toMatch :
			pos += 1
		if pos < len(self.FIELDS) and self.FIELDS[pos][0] == toMatch:
			self.fieldCB.set_active(pos)
		else:
			self.fieldCB.set_active(0)
			
		self.fieldCB.connect("changed", self.setUpAccordingToField)
		
		
		self.operatorCB = gtk.ComboBox()
		self.operatorCB.pack_start(cell)
		self.operatorCB.add_attribute(cell, "text", 1)
		
		self.entryWidget = gtk.Entry()
		self.spinBox = gtk.SpinButton(climb_rate=1)
		self.spinBox.set_increments(1, 2)
		self.spinBox.set_range(0, 5)
		
		deleteButton = gtk.ToolButton(gtk.STOCK_REMOVE)
		deleteButton.connect("clicked", self.destroy)
		
		self.pack_start(self.fieldCB, False)
		self.pack_start(self.operatorCB, False)
		self.pack_start(self.entryWidget)
		self.pack_start(self.spinBox)
		self.pack_end(deleteButton, False)
		
		self.show_all()
		glib.idle_add(self.setUpAccordingToField)
		
		
		
		if config[0] in ('playcount', 'rating'):
			operators = self.NUMERIC_OPERATORS
			self.spinBox.set_value(config[2])
		else:
			operators = self.OPERATORS
			self.entryWidget.set_text(str(config[2]))
		
		pos = 0
		toMatch = config[1]
		while pos < len(operators) and not operators[pos][0] == toMatch :
			pos += 1
		if pos < len(operators) and operators[pos][0] == toMatch:
			self.operatorCB.set_active(pos)
		else:
			self.operatorCB.set_active(0)

		
		
	def getConfig(self):
		field = self.FIELDS[self.fieldCB.get_active()][0]
		
		operatorKey = self.operatorCB.get_active_text()
		
		if field in ('playcount', 'rating'):
			value = self.spinBox.get_value()
		else:
			value = self.entryWidget.get_text()
		
		return field, operatorKey, value
		
		
	def setUpAccordingToField(self, cb=None, pos=None):
		if pos is None:
			pos = self.fieldCB.get_active()

		if self.FIELDS[pos][0] in ('playcount', 'rating'):
			self.entryWidget.hide()
			self.spinBox.show()
			self.operatorCB.set_model(self.NUMERIC_OPERATORSMODEL)
		else:
			self.entryWidget.show()
			self.spinBox.hide()
			self.operatorCB.set_model(self.OPERATORMODEL)
		

class CriterionManager(gtk.VBox):
	"""
		Widget to handle SQL Query creation with GUI
	"""
	def __init__(self):
		gtk.VBox.__init__(self)
		
		BB = gtk.HButtonBox()
		self.RB_Criterion = gtk.CheckButton(_("Whatever criterion matches"))
		self.RB_Random = gtk.CheckButton(_("Random order"))
		B_Add = gtk.ToolButton(gtk.STOCK_ADD)
		B_Add.connect("clicked", self.addCriterion)
		BB.add(self.RB_Criterion)
		BB.add(self.RB_Random)
		BB.add(B_Add)
		self.pack_end(BB, False)
		self.criterionBox = gtk.VBox()
		self.pack_start(self.criterionBox)
		
	def addCriterion(self, button=None, field=None, operator=None, condition=None):
		'''
			Adds a new criterion of selection (to be configured graphically), which will be processed upon validation
		'''
		
		self.criterionBox.pack_start(Criterion((field, operator, condition)), False)
		


		
	def getConfig(self):
		"""
			Return all parameter in dict
		"""
		config = {}
		if(self.RB_Random.get_active()):
			config['random'] = True
		else:
			config['random'] = False
			
		if(self.RB_Criterion.get_active()):
			config['link'] = ' OR '
		else:
			config['link'] = ' AND '
		
		config['criterions'] = []
		for crit in self.criterionBox:
			config['criterions'].append(crit.getConfig())
			
		return config
		
		
	def loadCriterions(self, dic):
		random = dic['random']
		logic_operator = dic['link']
		if(random):
			self.RB_Random.set_active(True)
		if(logic_operator == " OR "):
			self.RB_Criterion.set_active(True)
		
		crits = dic['criterions']
		for crit in crits:
			criterion = crit[0]
			operator = crit[1]
			condition = crit[2]
			self.addCriterion(None, criterion, operator, condition)
			
	def reset(self):
		self.RB_Random.set_active(False)
		self.RB_Criterion.set_active(False)
		for crit in self.criterionBox:
			crit.destroy()



class DynamicPlaylistCreator(gtk.Dialog):
	'''
		Create files containing parameters needed to create a SQL Query selecting specific tracks
	'''
	
	FOLDER = os.path.join(xdg.get_data_home(), 'playlists' + os.sep)
	DYNAMIC_FOLDER = FOLDER + os.sep + 'dynamic' + os.sep
	
	def __init__(self, name=None):
		
		self.E_Name = gtk.Entry()
		
		if(name == None):
			title = _("Add a dynamic playlist")
		else:
			title = _("Edit a dynamic playlist")
			self.E_Name.set_text(name)
			
		gtk.Dialog.__init__(self, title=title, buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
		
		Box_Main = self.get_content_area()
		Box_Name = gtk.HBox()
		L_Name = gtk.Label(_("Name") + " : ")
		Box_Name.pack_start(L_Name, False)
		Box_Name.pack_start(self.E_Name)
		
		
		Box_Main.pack_start(Box_Name, False)
		self.c_manager = CriterionManager()
		Box_Main.pack_start(self.c_manager)
		
		
		
		
		
		if(name == None):
			self.c_manager.addCriterion()
		else:
			self.loadCriterions(name)
			
		self.show_all()
		action = self.run()
		if(action == -3):
			self.validate(name)
		self.destroy()

	def loadCriterions(self, name):
		f = open(self.DYNAMIC_FOLDER + name,'r')
		data = f.readlines()
		f.close()
		self.c_manager.loadCriterions(eval(data[0]))
		
		
	def validate(self, edit):
		name = self.E_Name.get_text()
		f = open(self.DYNAMIC_FOLDER + name, 'w')
		f.write(str(self.c_manager.getConfig()))
		f.close()
		if(edit == None): # Not an edition, new one
			messager.emit('playlistAdded', self, ["dynamic", name])		
		
		
class TagsEditor(gtk.Dialog):
	'''
		Tags editor for music files
		TODO indicator to show what we're editing
	'''
	def __init__(self, data):
		gtk.Dialog.__init__(self, title=_("Tags editor"), flags=gtk.CAN_DEFAULT, buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_OK))

		single = type(data).__name__ == 'int';
		
		self.set_default_response(gtk.RESPONSE_OK)
		Box_Main = self.get_content_area()
		
		if single:
			Box_Main.pack_start(gtk.Label(_('Single file mod')))
		else:
			Box_Main.pack_start(gtk.Label(_('Container mod')))
		
		Box = gtk.Table(2,2)
		self.Table = Box
		
		self.i = 0
		
		self.tagsEntries = {}
		def add_line_for(tag, text=''):
			L = gtk.Label(_(tag) + " : ")
			Box.attach(L, 0, 1, self.i, self.i+1)
			self.tagsEntries[tag] = gtk.Entry()
			self.tagsEntries[tag].set_text(str(text))
			Box.attach(self.tagsEntries[tag], 1, 2, self.i, self.i+1)
			self.i += 1

		def add_line_forDEPREC(tag, text=''):
			Box = gtk.HBox()
			L = gtk.Label(_(tag) + " : ")
			self.tagsEntries[tag] = gtk.Entry()
			self.tagsEntries[tag].set_text(text)
			Box.pack_start(L)
			Box.pack_start(self.tagsEntries[tag])
			Box_Main.pack_start(Box)
			
		
		Box_Main.pack_start(Box)
		
		if type(data).__name__=='dict':
			self.incomingData = data
			
				
			self.cm_manager = CriterionManager()
			Box_Main.pack_start(gtk.HSeparator())
			
			Box_Main.pack_start(self.cm_manager)
		else:
			track_ID = data
			self.loadTrackData(track_ID)
			
		if(self.incomingData != None):
			for key in self.incomingData.iterkeys():
				add_line_for(key, self.incomingData[key])
		
		
		self.show_all()
		reponse = self.run()
		if(reponse == gtk.RESPONSE_OK):
			self.validate()
		self.destroy()
		
	def loadTrackData(self, track_ID):
		self.track = elements.Track(track_ID)
		fichier = self.track.path
		
		self.incomingData = self.track.get_tags()
		
		L = gtk.Label(_('Path') + " : " + fichier)
		L.set_selectable(True)
		self.Table.attach(L, 0, 2, self.i, self.i+1)
		self.i += 1
		
	def validate(self):
		#FIXME optimize process by saving file only once (only where all tags are processed)
		try:
			matchingTracks = (self.track,)
		except AttributeError:
			matchingTracks = elements.db.get_tracks(self.incomingData, self.cm_manager.getConfig())
		for track in matchingTracks:
			for key in self.incomingData.iterkeys():
				if(self.tagsEntries[key].get_text() != self.incomingData[key]):
					track.set_tag(key, self.tagsEntries[key].get_text())
		

class SettingsEditor(gtk.Dialog):
	def __init__(self, section='general'):
		gtk.Dialog.__init__(self, title=_("Settings editor"), buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
		self.Box_Main = gtk.HBox()
		self.get_content_area().pack_start(self.Box_Main)
		
		icons_dir = xdg.get_data_dir() + 'icons/'
		TreeView = gtk.TreeView()
		TreeView.connect("button-press-event", self.on_section_click)
		self.sectionModel = gtk.TreeStore(str, str, gtk.gdk.Pixbuf)
		self.sectionModel.append(None, ['general', _('General'), None])
		self.sectionModel.append(None, ['folders', _('Indexed folders'), None])
		music_node = self.sectionModel.append(None, ['music', _('Music'), gtk.gdk.pixbuf_new_from_file(icons_dir + 'track.png')])
		self.sectionModel.append(music_node, ['music_filters', _('Filters'), None])
		self.sectionModel.append(music_node, ['audioscrobbler', _('Audioscrobbler'), None])
		self.sectionModel.append(None, ['pictures', _('Pictures'), None])
		self.sectionModel.append(None, ['videos', _('Videos'), None])
		
		cell = gtk.CellRendererText()
		cellpb = gtk.CellRendererPixbuf()
		Col_Label = gtk.TreeViewColumn(_('Section'), cellpb, pixbuf=2)
		Col_Label.pack_start(cell, False)
		Col_Label.set_attributes(cell, text=1)
		TreeView.append_column(Col_Label)
		TreeView.set_model(self.sectionModel)
		self.Box_Main.pack_start(TreeView, False)
		
		self.show_all()
		self.setupWidgets()
		self.load_section(section)
		
		reponse = self.run()
		if(reponse == -3):
			self.validate()
		self.destroy()
	
	def createUCWidgetFor(self, module):
		"""
			Create settings section for standard UC module
		"""
		Box = UCSettingsWidget(module)

		self.Box_Main.pack_start(Box)
		self.widgets[module] = Box
	
	def setupWidgets(self):
		self.widgets = {}
		
		# --- General section ---
		Box_General = gtk.VBox()
		
		#self.CB_gui_framework = gtk.combo_box_new_text()
		#libs = {'Gtk 2':0, 'Qt 4':1}
		#self.CB_gui_framework.append_text('Gtk 2')
		#self.CB_gui_framework.append_text('Qt 4')
		
		self.picture_enabled = gtk.CheckButton(_('Enable pictures manager'))
		self.picture_enabled.set_active(settings.get_option('pictures/enabled', False))
		self.video_enabled = gtk.CheckButton(_('Enable videos manager'))
		self.video_enabled.set_active(settings.get_option('videos/enabled', False))
		

		
		#Box_General.pack_start(self.CB_gui_framework)
		Box_General.pack_start(self.picture_enabled)
		Box_General.pack_start(self.video_enabled)
		self.Box_Main.pack_start(Box_General)
		self.widgets['general'] = Box_General
		
		# --- Folders section ---
		Box_folders = gtk.VBox()
		TV_folders = gtk.TreeView()
		self.folders = gtk.ListStore(str, bool)
		cell = gtk.CellRendererText()
		Col_Label = gtk.TreeViewColumn(_('Folders'), cell, text=0)
		cell_toggle = gtk.CellRendererToggle()
		Col_Recursive = gtk.TreeViewColumn(_('Dig'), cell_toggle, active=1)
		cell_toggle.connect('toggled', self.toggle_recursive_folder)
		TV_folders.append_column(Col_Label)
		TV_folders.append_column(Col_Recursive)
		TV_folders.set_model(self.folders)
		
		for folder in settings.get_option('music/folders', []):
			self.folders.append([folder[0], folder[1]])
		
		def add_folder(*args):
			dialog = gtk.FileChooserDialog(action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, buttons=(gtk.STOCK_OPEN, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER))
			dialog.set_current_folder_uri('~')
			dialog.run()
			fichier = dialog.get_filename()
			dialog.destroy()
			self.folders.append([fichier, False])
			
		def remove_folder(*args):
			try:
				model, iter = TV_folders.get_selection().get_selected()
				model.remove(iter)
			except TypeError:
				warning = gtk.MessageDialog(self, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_CLOSE, message_format=_('You must select a folder before clicking the remove button'))
				warning.run()
				warning.destroy()
		
		B_add_folder = gtk.Button(_('Add'), gtk.STOCK_ADD)
		B_add_folder.connect('clicked', add_folder)
		B_remove_folder = gtk.Button(_('Remove'), gtk.STOCK_REMOVE)
		B_remove_folder.connect('clicked', remove_folder)
		Box_folders.pack_start(TV_folders)
		Box = gtk.HBox()
		Box.pack_start(B_add_folder)
		Box.pack_start(B_remove_folder)
		Box_folders.pack_start(Box, False)
		
		self.Box_Main.pack_start(Box_folders)
		self.widgets['folders'] = Box_folders
		
		
		# ******* Music section *********
		
		Box_music = gtk.Table(2,2)
		i = 0
		Box_music.attach(gtk.Label(_('Playback library') + ' :'), 0, 1, i, i+1, gtk.FILL|gtk.EXPAND, gtk.FILL)
		self.CB_music_playback_lib = gtk.combo_box_new_text()
		libs = {'GStreamer':0, 'MPlayer':1, 'VLC':2}
		self.CB_music_playback_lib.append_text('GStreamer')
		self.CB_music_playback_lib.append_text('MPlayer')
		self.CB_music_playback_lib.append_text('VLC')
		
		try:
			self.CB_music_playback_lib.set_active(libs[settings.get_option('music/playback_lib', 'GStreamer')])
		except KeyError:
			self.CB_music_playback_lib.set_active(libs['GStreamer'])
		Box_music.attach(self.CB_music_playback_lib, 1, 2, i, i+1, gtk.FILL, gtk.FILL)
		
		
		i += 1
		Box_music.attach(gtk.Label(_('Panel icon size') + ' :'), 0, 1, i, i+1, gtk.FILL, gtk.FILL)
		self.CB_icon_size_panel_music = gtk.combo_box_new_text()
		self.CB_icon_size_panel_music.append_text('16')
		self.CB_icon_size_panel_music.append_text('24')
		self.CB_icon_size_panel_music.append_text('32')
		self.CB_icon_size_panel_music.append_text('48')
		self.CB_icon_size_panel_music.append_text('64')
		
		self.CB_icon_size_panel_music.set_active(settings.get_option('music/panel_icon_size', 32) / 16)
		Box_music.attach(self.CB_icon_size_panel_music, 1, 2, i, i+1, gtk.FILL|gtk.EXPAND, gtk.FILL)
		
		i+= 1
		self.usePerformerTag = gtk.CheckButton(_('Show performer instead of artist in library browser'))
		self.usePerformerTag.set_active(settings.get_option('music/use_performer', False))
		Box_music.attach(self.usePerformerTag, 0, 2, i, i+1, gtk.FILL, gtk.FILL)
		
		self.Box_Main.pack_start(Box_music)
		self.widgets['music'] = Box_music
		
		
		self.Box_mfilters = FilterManager(settings.get_option('music/filters', {}))
		
		
		
		self.Box_Main.pack_start(self.Box_mfilters)
		self.widgets['music_filters'] = self.Box_mfilters
		
		# Audioscrobbler section
		Box_audioscrobbler = gtk.Table(2, 2)
		Box_audioscrobbler.attach(gtk.Label(_('Login') + ' : '), 0, 1, 0, 1, gtk.FILL, gtk.FILL) 
		
		self.audioscrobbler_login = gtk.Entry()
		self.audioscrobbler_login.set_text(settings.get_option('music/audioscrobbler_login', ''))
		Box_audioscrobbler.attach(self.audioscrobbler_login, 1, 2, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL) 
		
		
		Box_audioscrobbler.attach(gtk.Label(_('Password') + ' : '), 0, 1, 1, 2, gtk.FILL, gtk.FILL) 
		self.audioscrobbler_password = gtk.Entry()
		self.audioscrobbler_password.set_text(settings.get_option('music/audioscrobbler_password', ''))
		Box_audioscrobbler.attach(self.audioscrobbler_password, 1, 2, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL) 
		
		self.Box_Main.pack_start(Box_audioscrobbler)
		self.widgets['audioscrobbler'] = Box_audioscrobbler
		
		
		# UC sections
		self.createUCWidgetFor('pictures')

		
		self.createUCWidgetFor('videos')
		
		# Videos additional settings
		i = 3
		self.widgets['videos'].attach(gtk.Label(_('Playback library') + ' :'), 0, 1, i, i+1, gtk.FILL, gtk.FILL)
		self.CB_video_playback_lib = gtk.combo_box_new_text()
		libs = {'GStreamer':0, 'MPlayer':1, 'VLC':2}
		self.CB_video_playback_lib.append_text('GStreamer')
		self.CB_video_playback_lib.append_text('MPlayer')
		self.CB_video_playback_lib.append_text('VLC')
		
		try:
			self.CB_video_playback_lib.set_active(libs[settings.get_option('videos/playback_lib', 'GStreamer')])
		except KeyError:
			self.CB_video_playback_lib.set_active(libs['GStreamer'])
		self.widgets['videos'].attach(self.CB_video_playback_lib, 1, 2, i, i+1, gtk.FILL, gtk.FILL)
		
		
		
	def load_section(self, section):
		try:
			self.active_widget.hide_all()
		except:
			pass
		self.active_widget = self.widgets[section]
		self.active_widget.show_all()
		
	def on_section_click(self, TreeView, event):
		if event.button == 1:
			path = TreeView.get_path_at_pos(int(event.x),int(event.y))[0]
			if (path != None):
				section = self.sectionModel[path][0]
				self.load_section(section)
				
	def validate(self):
		settings.set_option('pictures/enabled', self.picture_enabled.get_active())
		settings.set_option('videos/enabled', self.video_enabled.get_active())
		
		folders = []
		iter = self.folders.get_iter_first()
		while iter is not None:
			folders.append((self.folders.get_value(iter, 0), self.folders.get_value(iter, 1)))
			iter = self.folders.iter_next(iter)
		settings.set_option('music/folders', folders)
			
		#Music settings:
		settings.set_option('music/playback_lib', self.CB_music_playback_lib.get_active_text())
		settings.set_option('music/panel_icon_size', int(self.CB_icon_size_panel_music.get_active_text()))
		settings.set_option('music/use_performer', self.usePerformerTag.get_active())
		
		settings.set_option('music/filters', self.Box_mfilters.getConfig())
		

		#Audioscrobbler settings :
		settings.set_option('music/audioscrobbler_login', self.audioscrobbler_login.get_text())
		settings.set_option('music/audioscrobbler_password', self.audioscrobbler_password.get_text())
		
		
		# UC modules settings
		for ucModule in ('pictures', 'videos'):
			self.widgets[ucModule].save()
		
		#Videos additional settings
		settings.set_option('videos/playback_lib', self.CB_video_playback_lib.get_active_text())
		

		settings.MANAGER.save()
		
	def toggle_recursive_folder(self, cell, path):
		iter = self.folders.get_iter((int(path),))
		val = self.folders.get_value(iter, 1)
	
		# toggle the value
		val = not val
	
		# set new value
		self.folders.set(iter, 1, val)
			
class UCSettingsWidget(gtk.Table):
	def __init__(self, module):
		gtk.Table.__init__(self)
		
		self.module = module
		i = 0
		self.attach(gtk.Label(_('Indexed extensions') + ' :'), 0, 1, i, i+1)
		self.extensions = gtk.Entry()
		self.extensions.set_text(settings.get_option(module + '/indexed_extensions', ''))
		self.attach(self.extensions, 1, 2, i, i+1)
		i += 1
		self.preload = gtk.CheckButton(_('Preload this module'))
		self.preload.set_active(settings.get_option(module + '/preload', False))
		self.preload.set_tooltip_text(_('If true, will be preloaded before application start, else will be loaded on demand (slightly faster startup)'))
		self.attach(self.preload, 0, 2, i, i+1)
		i += 1
		self.attach(gtk.Label(_('Panel thumbnail size') + ' :'), 0, 1, i, i+1)
		self.panelIconSizeCB = gtk.combo_box_new_text()
		self.panelIconSizeCB.append_text('16')
		self.panelIconSizeCB.append_text('24')
		self.panelIconSizeCB.append_text('32')
		self.panelIconSizeCB.append_text('48')
		self.panelIconSizeCB.append_text('64')
		
		self.panelIconSizeCB.set_active(settings.get_option(module + '/panel_icon_size', 32) /16)
		self.attach(self.panelIconSizeCB, 1, 2, i, i+1)
		
		
	def save(self):
		settings.set_option(self.module + '/indexed_extensions', self.extensions.get_text())
		settings.set_option(self.module + '/preload', self.preload.get_active())
		settings.set_option(self.module + '/panel_icon_size', int(self.panelIconSizeCB.get_active_text()))
		
	
class FilterManager(gtk.VBox):
	"""
		Widget that handles filter managment with a CriterionManager
	"""
	def __init__(self, config):
		gtk.VBox.__init__(self)
		self.config = config
		self.CB_filters = gtk.combo_box_new_text()
		self.CB_filters.connect('changed', self.load_filter)
		B_add = gtk.Button('', gtk.STOCK_ADD)
		B_add.connect('clicked', self.add_filter)
		B_remove = gtk.Button('', gtk.STOCK_REMOVE)
		B_remove.connect('clicked', self.deleteFilter)
		self.RB_enabled = gtk.CheckButton(_("Enabled"))
		Box = gtk.HBox()
		Box.pack_start(self.RB_enabled, False)
		Box.pack_start(self.CB_filters)
		Box.pack_start(B_add, False)
		Box.pack_start(B_remove, False)
		self.pack_start(Box, False)
		
		
		self.c_manager = CriterionManager()
		self.pack_start(self.c_manager)
		
		for key in self.config.iterkeys():
			self.CB_filters.append_text(key)
		
		try:
			self.CB_filters.set_active(0)
			self.active_filter = self.CB_filters.get_active_text()
		except:
			pass
		
	
		
	def add_filter(self, button=None):
		DN = gtk.Dialog()
		Entry = gtk.Entry()
		hbox = gtk.HBox()
		hbox.pack_start(gtk.Label(_("Name") + " : "), False, 5, 5)
		hbox.pack_end(Entry)
		DN.vbox.add(hbox)
		
		DN.add_button('OK', 1)
		DN.add_button(_("Cancel"), 0)
		DN.show_all()
		action = DN.run()
		label = Entry.get_text()
		DN.destroy()
		if(action == 1):
			self.CB_filters.append_text(label)
			self.CB_filters.set_active(self.count)
			self.count += 1
			
	def deleteFilter(self, button=None):
		filterName = self.CB_filters.get_active_text()
		del self.config[filterName]
		self.activeFilter = None
		activeIndex = self.CB_filters.get_active()
		self.CB_filters.remove_text(activeIndex)
		self.CB_filters.set_active(activeIndex)
			
	def getConfig(self):
		"""
			Return a dict with all filters configuration for this model
		"""
		filter = self.CB_filters.get_active_text()
		if(filter != None): # saving current
			self.config[filter] = self.c_manager.getConfig()
			self.config[filter]['enabled'] = self.RB_enabled.get_active()
		return self.config
	
	
	def load_filter(self, widget):
		try:
			if(self.active_filter != None): # saving previous
				self.config[self.active_filter] = self.c_manager.getConfig()
				self.config[self.active_filter]['enabled'] = self.RB_enabled.get_active()
		except:
			pass
			
		filter = self.CB_filters.get_active_text()
		self.active_filter = filter
		self.c_manager.reset()
		self.RB_enabled.set_active(False)
		try:
			self.c_manager.loadCriterions(self.config[filter])
			if(self.config[filter]['enabled']):
				self.RB_enabled.set_active(True)
		except KeyError:
			pass

class ImportHelper(gtk.Dialog):
	'''
		Dialog used to retrieve data from another Bullseye database
	'''
	def __init__(self, db):
		gtk.Dialog.__init__(self, title=_("Import helper"), buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
		self.Box_Main = gtk.VBox()
		self.get_content_area().pack_start(self.Box_Main)
		self.button = gtk.FileChooserButton(_("Choose a database file"))
		self.Box_Main.pack_start(self.button)
		
		self.c_manager = CriterionManager()
		self.Box_Main.pack_start(self.c_manager)
		
		self.show_all()
		response = self.run()
		if(response == gtk.RESPONSE_ACCEPT):
			db.retrieveFromSave(self.button.get_filename(), self.c_manager.getConfig())
		self.destroy()
		
		
		
class UCStructureHelper(gtk.Dialog):
	'''
		Dialog used to create a folders strucutre and move all files of a UC Module into it
	'''
	def __init__(self, module):
		self.module = module
		gtk.Dialog.__init__(self, title=_("Preparing for files moving"), buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
		self.Box_Main = gtk.VBox()
		self.get_content_area().pack_start(self.Box_Main)
		self.button = gtk.FileChooserButton(gtk.FileChooserDialog(_("Choose the structure root folder"), 
			self, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, (gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK)))
			
		self.button.set_filename(settings.get_option(module + '/structure_folder', '~/' + module + '/Bullseye'))
		self.Box_Main.pack_start(self.button)
		
		CB = gtk.ComboBox()
		self.Box_Main.pack_start(CB)
		
		self.showAntagonistic = gtk.CheckButton(_("Show antagonistic"))
		self.Box_Main.pack_start(self.showAntagonistic)
		
		#self.c_manager = CriterionManager()
		#self.Box_Main.pack_start(self.c_manager)
		
		self.show_all()
		
	def run(self):
		if gtk.Dialog.run(self) == gtk.RESPONSE_ACCEPT:
			folder = self.button.get_filename()
			settings.set_option(self.module + '/structure_folder', folder)
			self.destroy()
			return folder
		else:
			self.destroy()
			return None