# -*- coding: utf-8 -*-
import gtk
import os
from common import messager, settings, xdg
from abstract.musicpanel import LibraryPanelInterface
from gui import menus, modales
import glib
import threading
import time
#from gui.menus import TrackMenu, TrackContainerMenu

icon_track = gtk.gdk.pixbuf_new_from_file('icons/track.png')
icon_artist = gtk.gdk.pixbuf_new_from_file('icons/artist.png')
icon_album = gtk.Image().render_icon(gtk.STOCK_CDROM, gtk.ICON_SIZE_MENU)
icon_genre = gtk.gdk.pixbuf_new_from_file('icons/genre.png')
icon_year = gtk.gdk.pixbuf_new_from_file('icons/year.png')
icon_rating = gtk.gdk.pixbuf_new_from_file('icons/star.png')
icons = {"title":icon_track, "artist":icon_artist, "album":icon_album, "genre":icon_genre, "rating":icon_rating, "year":icon_year}
icon_size = settings.get_option('music/panel_icon_size', 32)

class LibraryPanel(LibraryPanelInterface, gtk.VBox):
	"""
		Burn factor indicates container with playcount concentrated on fews children
	"""
	def __init__(self, mdb, queueManager):
		self.mdb = mdb
		self.queueManager = queueManager
		self.tracks = {}
		self.TreeView = gtk.TreeView()
		#self.TreeView.set_headers_visible(False)
		self.TreeView.enable_model_drag_source(gtk.gdk.MODIFIER_MASK,  [('text/plain', 0, 0)] ,  gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE)
		self.TreeView.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
		#self.TreeView.set_rubber_banding(True) # multiple selection with dragging
		self.TreeView.connect('row-activated', self.enqueue)
		self.TreeView.connect("drag-begin", self.keepTrueSelection)
		self.TreeView.connect("button-press-event", self.on_button_press)
		self.TreeView.connect("drag-data-get", self.on_drag_data_get)

		self.mode = '("artist", "album", "title")'
		self.model = gtk.TreeStore(gtk.gdk.Pixbuf, int, str, int, int, float, str, bool, str) # icon, id, label, playcount, rating, burn, rounded_burn, is_separator, type
		

		
		def is_separator(model, iter):
			return model.get_value(iter, 7)
		self.TreeView.set_row_separator_func(is_separator)
		
		#self.TreeView.set_rules_hint(True)
		self.TreeView.set_model(self.model)
		
		pb = gtk.CellRendererPixbuf()
		#self.Col_Icon = BSColumn('col_icon_panel', _('Icon'), pb, pixbuf=0, default_width=80)
		self.Col_Icon = BSColumn('col_icon_panel', _('Icon'), default_width=80)
		self.Col_Icon.pack_start(pb, False)
		self.Col_Icon.set_attributes(pb, pixbuf=0)
		Col_Label = BSColumn('col_label_panel', _('Label'), expand=True, default_width=170)
		self.TreeView.append_column(self.Col_Icon)
		self.TreeView.append_column(Col_Label)
		cell = gtk.CellRendererText()
		
		#Col_Label.pack_start(pb, False)
		Col_Label.pack_start(cell, True)
		#Col_Label.set_attributes(pb, pixbuf=0)
		Col_Label.add_attribute(cell, 'text', 2)
		Col_Label.set_sort_column_id(2)
		Col_Label.connect("clicked", self.on_column_clicked)
		Col_Count = BSColumn('col_count_panel', _('Count'), cell, model_text=3, default_width=50)
		Col_Rating = gtk.TreeViewColumn(_('Rating'), cell, text=4)
		Col_Valor = gtk.TreeViewColumn(_('Burn'), cell, text=6)
		self.TreeView.append_column(Col_Count)
		self.TreeView.append_column(Col_Rating)
		self.TreeView.append_column(Col_Valor)
		Col_Count.set_sort_column_id(3)
		Col_Count.connect("clicked", self.on_column_clicked)
		Col_Rating.set_sort_column_id(4)
		Col_Rating.connect("clicked", self.on_column_clicked)
		Col_Valor.set_sort_column_id(5)
		Col_Valor.connect("clicked", self.on_column_clicked)
		
		self.notLoading = threading.Event()
		self.notLoading.set()
		self.fillModel()
		
		LS_CB = gtk.ListStore(str, str)
		LS_CB.append(['("artist", "album", "title")', _("Artist")])
		LS_CB.append(['("album", "title")', _("Album")])
		LS_CB.append(['("rating", "title")', _("Rating")])
		LS_CB.append(['("genre", "album", "title")', _("Genre")])
		LS_CB.append(['("year", "artist", "album", "title")', _("Year")])
		LS_CB.append(['("year", "genre", "artist", "album", "title")', _("Year - Genre")])
		LS_CB.append(['("rating", "year", "genre", "artist", "album", "title")', _("Rating - Year - Genre")])
		CB = gtk.ComboBox()
		cell = gtk.CellRendererText()
		CB.pack_start(cell)
		CB.add_attribute(cell, "text", 1)
		CB.set_model(LS_CB)
		CB.set_active(0)
		CB.connect("changed", self.changeMode)
		self.CB = CB
		
		B_refresh = gtk.ToolButton(gtk.STOCK_REFRESH)
		B_refresh.connect('clicked', self.fillModel, True)
		
		gtk.VBox.__init__(self)
		Box_mode = gtk.HBox()
		Box_mode.pack_start(CB)
		Box_mode.pack_start(B_refresh, False)
		self.pack_start(Box_mode, False)
		SW = gtk.ScrolledWindow()
		SW.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		SW.add(self.TreeView)
		self.pack_start(SW)
		E_Search  = gtk.Entry()
		Search = SearchEntry(E_Search)
		Search.connect('activate', self.searchFor)
		self.pack_start(E_Search, False)
	

		
		
	def addExpandPath(self, model, node):
		self.expand_paths.append(model.get_path(node))
		
		
	def append(self, model, data, parentNode):
		def getIcon(type, label):
			try:
				path = os.path.join(xdg.get_thumbnail_dir(type + '/medium'),  label.replace ('/', ' ')) # + '.jpg')
				icon = gtk.gdk.pixbuf_new_from_file_at_size(path, icon_size, icon_size)
			except:
				icon = icons[type]
			return icon
		data[0] = getIcon(data[8], data[2])
		node = self.model.append(parentNode, data)
		return node
		
	def beginInsertRows(self, model, index):
		return
		
	def clear(self, model):
		model.clear()
		
	def changeMode(self, model):
		self.mode = self.CB.get_active_text()
		self.fillModel()
		
	def endInsertRows(self, model, expand_paths):
		for path in expand_paths:
			self.TreeView.expand_row(path, False)
			
			
	def enqueue(self, w, i, c):
		mode = eval(self.CB.get_active_text()) #Ex : '("artist", "album")'
		level = len(i) # Deep level of selection (ex : 1 = Artiste, 2 = Album, 3 = Title)
		
		dic = {}
		while(level > 0):
			i = i[0:level]
			level -= 1
			dic[mode[level]] = self.model[i][2]

		self.queueManager.addSelection(self.mdb.getTracks(dic))
	
	
	
	
	def keepTrueSelection(self, TV, drag_context):
		'''
			Trick function to drag multiple itmes because once you start to drag, selection become the dragged item
		'''
		selection = TV.get_selection()
		if(selection.get_selected_rows()[1][0] in self.selection):
			for path in self.selection:
				selection.select_path(path)
		else:
			self.selection = selection.get_selected_rows()[1]
		
	def on_button_press(self, TV, event):
		if event.button == 1: # Left click
			model, self.selection = TV.get_selection().get_selected_rows()
		elif event.button == 3: # Right click
			path = TV.get_path_at_pos(int(event.x),int(event.y))[0]
			if (path != None):
				mode = eval(self.mode)
				model = TV.get_model()
				if(len(path) != len(mode)): # Container
					dic = {}
					i = 0
					while (i != len(path)):
						dic[mode[i]] = model[path[0:i+1]][2]
						i += 1
					m = menus.TrackContainerMenu(dic)
					m.show_all()
					m.popup(None, None, None, event.button, event.time)
				else: # Track
					track_ID = model[path][1]
					ligne = model.get_iter(path)
					m = menus.TrackMenu(track_ID)
					m.append(gtk.SeparatorMenuItem())
					i = gtk.ImageMenuItem(_("Remove from queue"))
					i.set_image(gtk.image_new_from_stock(gtk.STOCK_REMOVE, gtk.ICON_SIZE_MENU))
					#i.connect('activate', self.enlever_piste, ligne)
					m.append(i)

					i = gtk.ImageMenuItem(_("Add to direct list"))
					i.set_image(gtk.image_new_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU))
					m.append(i)
					m.show_all()
					m.popup(None, None, None, event.button, event.time)
		elif event.button == 2:
			path = TV.get_path_at_pos(int(event.x),int(event.y))[0]
			if(TV.row_expanded(path)):
				TV.collapse_row(path)
			else:
				TV.expand_row(path, False)
	
	def on_column_clicked(self, column):
		def disable_sorting_state():
			time.sleep(2.0)
			self.model.set_sort_column_id(-2, 0)
			
		a = threading.Thread(target=disable_sorting_state)
		a.start()
		
	def on_drag_data_get(self, widget, drag_context, selection_data, info, timestamp):
		def get_dic(path, mode):
			# Return a dict containing all data necessary to identify the dragged item
			level = len(path)
			dic = {}
			while (level > 0):
				dic[mode[level -1]] = self.model[path[0:level]][2]
				level -= 1
			return dic

		model, paths = widget.get_selection().get_selected_rows() # Is the index of the single clicked item (auto-selected by button-press event)
		if paths[0] in self.selection: # We check that this index is not present in the previous multiple selection
			paths = self.selection # If it's the case, it means that we wanted to DND the multiple selection and not the single clicked item
		T = []
		mode = eval(self.mode)
		for path in paths:
			T.append(get_dic(path, mode))
		selection_data.set_text(str(T))

		
	def searchFor(self, entry):
		text = entry.get_text()
		mode = self.CB.get_active_text()
		e = threading.Event()
		self.fillModel(word=text)
		
		
class PlaylistBrowser(gtk.VBox):
	
	PLAYLIST_ICON_PATH = xdg.get_data_dir() + 'icons' + os.sep + 'playlist.png'
	FOLDER_ICON_PATH = xdg.get_data_dir() + 'icons' + os.sep + 'folder.png'
	
	FOLDER = os.path.join(xdg.get_data_home(), 'playlists' + os.sep)
	DYNAMIC_FOLDER = FOLDER + os.sep + 'dynamic' + os.sep
	
	def __init__(self, db, queueManager):
		# --- Data attributes ---
		self.mdb = db
		self.queueManager = queueManager
		
		self.TV = gtk.TreeView()
		self.model = gtk.TreeStore(gtk.gdk.Pixbuf, int, str) # icon, ID, title
		
		
		pixbuf_dir = gtk.ToolButton().render_icon(gtk.STOCK_DIRECTORY, gtk.ICON_SIZE_BUTTON)
		self.staticNode = self.model.append(None, [pixbuf_dir, 0, "Static playlists"])
		for f in os.listdir(self.FOLDER):
			pixbuf = gtk.gdk.pixbuf_new_from_file(self.PLAYLIST_ICON_PATH)
			if os.path.isfile(os.path.join(self.FOLDER, f)):
				self.model.append(self.staticNode, [pixbuf, 0, f])
		
		self.dynamicNode = self.model.append(None, [pixbuf_dir, 0, _("Dynamic playlists")])

		for f in os.listdir(self.DYNAMIC_FOLDER):
			if os.path.isfile(os.path.join(self.DYNAMIC_FOLDER, f)):
				self.model.append(self.dynamicNode, [pixbuf, 0, f])
		
				
				
		
		self.TV.set_model(self.model)
		column = gtk.TreeViewColumn('Column 0')
		self.TV.append_column(column)
		cell = gtk.CellRendererText()
		pb = gtk.CellRendererPixbuf()
		column.pack_start(pb, False)
		column.pack_start(cell, True)
		column.add_attribute(cell, 'text', 2)
		column.set_attributes(pb, pixbuf=0)
		
		self.TV.expand_all()
		
		# --- Signals ---
		self.TV.connect('row-activated', self.onActivated)
		self.TV.connect("button-press-event", self.onButtonPress)
		
		# Messager registrations
		messager.connect(self.addPlaylist, 'playlistAdded')
		gtk.VBox.__init__(self)
		self.pack_start(self.TV)
		
	
	def addPlaylist(self, data):
		label = data[1]
		type = data[0]
		if(type == "dynamic"):
			parent = self.dynamicNode
		else:
			parent = self.staticNode
			
		pixbuf = gtk.gdk.pixbuf_new_from_file(self.PLAYLIST_ICON_PATH)
		self.model.append(parent, [pixbuf, 0, label])
	
	

	
	def edit(self, iter):
		path = self.model.get_path(iter)
		name = self.model.get_value(iter, 2)
		if(path[0] == 0): # static playlist
			print('temp rename')
		else: # dynamic playlist
			d = modales.DynamicPlaylistCreator(name)
		
		
	def deletePlaylist(self, iter):
		path = self.model.get_path(iter)
		name = self.model.get_value(iter, 2)
		if(path[0] == 0):
			os.remove(self.FOLDER + name)
		else:
			os.remove(self.DYNAMIC_FOLDER + name)
		self.model.remove(iter)
		
	def onActivated(self, w, i, column, stop_mod=False):
		level = len(i)
		if(level == 2): #On est bien sur une playlist
			name = self.model[i][2]
			if(i[0] == 0): #On est sur une playlist perso
				f = open(self.FOLDER + os.sep + name, 'r')
				IDList = f.readlines()
				f.close()
				self.queueManager.addPlaylist(name, IDList)
			else: #On est sur une playlist intelligente
				fichier = open(self.DYNAMIC_FOLDER + name,'r')
				data = fichier.readlines()
				fichier.close()
				self.queueManager.addQueue()
				self.queueManager.addSelection(self.mdb.getDynamicListTracks(eval(data[0])))
		
		
	def onButtonPress(self, TreeView, event):
		# Right click
		if event.button == 3:
			path = TreeView.get_path_at_pos(int(event.x),int(event.y))[0]
			level = len(path)
			if(level ==2):
				ligne = self.model.get_iter(path)
				m = menus.PlaylistMenu(self, ligne)
				#i.connect('activate', self.supprimer, playlist)
				#i.show() 
				#m.append(i) 
				m.popup(None, None, None, event.button, event.time)
			else:
				m = menus.CreatePlaylistMenu()
				m.popup(None, None, None, event.button, event.time)
				






class SearchEntry(object):
	"""
		A gtk.Entry that emits the "activated" signal when something has
		changed after the specified timeout
	"""
	def __init__(self, entry=None, timeout=500):
		"""
		Initializes the entry
		"""
		self.entry = entry
		self.timeout = timeout
		self.change_id = None

		if self.entry is None:
			self.entry = gtk.Entry()

		#self.entry.connect('changed', self.on_entry_changed)
		self.entry.connect('icon-press', self.on_entry_icon_press)

	def on_entry_changed(self, *e):
		"""
		Called when the entry changes
		"""
		if self.change_id:
			glib.source_remove(self.change_id)

		self.change_id = glib.timeout_add(self.timeout, self.entry_activate)

	def on_entry_icon_press(self, entry, icon_pos, event):
		"""
		Clears the entry
		"""
		self.entry.set_text('')

	def entry_activate(self, *e):
		"""
		Emit the activate signal
		"""
		self.entry.activate()

	def __getattr__(self, attr):
		"""
		Tries to pass attribute requests
		to the internal entry item
		"""
		return getattr(self.entry, attr)
		
class BSColumn(gtk.TreeViewColumn):
	'''
		Bullseye Special Column
	'''
	def __init__(self, name, label, cell=None, model_text=None, pixbuf=None, default_width=50, expand=False):
		kwargs = {}
		if(model_text is not None):
			kwargs['text'] = model_text
		if(pixbuf is not None):
			kwargs['pixbuf'] = pixbuf
		gtk.TreeViewColumn.__init__(self, label, cell, **kwargs)
		self.name = name
		width = settings.get_option('music/' + self.name + '_width', default_width)
		self.set_min_width(default_width)
		#self.set_max_width(width)
		self.set_expand(expand)
		self.set_resizable(True)
		self.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
		self.connect('notify::width', self.on_width_change)
		
	def on_width_change(self, *args):
		settings.set_option('music/' + self.name + '_width', self.get_width())