# -*- coding: utf-8 -*-
import gtk
import glib
import threading
import time
import gobject
import gettext
import logging
import os

from operator import attrgetter


from common import messager, settings, util, xdg

from data.elements import Track, DB

import gui.modales
from gui.menus import TrackMenu
from gui.util import icons, etoiles

IM = icons.IconManager()

logger = logging.getLogger(__name__)

class QueueManager(gtk.VBox):
	"""
		~ a gtk.NoteBook (TabWidget) where each tab is a Queue containing tracks
		TODO button stop = set stop track 
	"""
	def __init__(self, playerWidget):
		#gtk.rc_parse_string("style \"ephy-tab-close-button-style\"\n"
			     #"{\n"
			       #"GtkWidget::focus-padding = 0\n"
			       #"GtkWidget::focus-line-width = 0\n"
			       #"xthickness = 0\n"
			       #"ythickness = 0\n"
			     #"}\n"
			     #"widget \"*.ephy-tab-close-button\" style \"ephy-tab-close-button-style\"")
		gtk.VBox.__init__(self)
		self.NoteBook = gtk.Notebook()
		
		self.playerWidget = playerWidget
		self.playerWidget.queueManager = self



		self.loadState()
		self.setupShortcuts()
		glib.timeout_add_seconds(300, self.save_state)
		
		actionBox = gtk.HBox()
		scrollToCurrentButton = gtk.ToolButton(gtk.STOCK_JUMP_TO)
		scrollToCurrentButton.connect('clicked', self.scrollToCurrent)
		searchEntry = gtk.Entry()
		searchEntry.connect('activate', self.filter)
		actionBox.pack_start(scrollToCurrentButton, False)
		actionBox.pack_start(searchEntry)
		
		self.pack_start(self.NoteBook)
		self.pack_start(actionBox, False)
			
		self.NoteBook.connect('expose-event', self.redrawAddTabButton)
		self.NoteBook.connect('button-release-event', self.onButtonRelease)
		

	

	def getAddTabButtonPos(self):
		'''
			This function and the following twos are meant for displaying the new tab button
		'''
		try:
			last_tab_label = self.NoteBook.get_tab_label(self.NoteBook.get_nth_page(self.NoteBook.get_n_pages() -1))
			alloc = last_tab_label.get_allocation()
		except:
			print 'TODO'
		
		return (alloc.x + alloc.width + 10, alloc.y + 4)
			
	def onButtonRelease(self, widget, event):
		if(event.button == 1):
			x, y = self.getAddTabButtonPos()
			x_root, y_root = self.window.get_root_origin()
			x = x + x_root - 5
			y = y + y_root + 16
			if(event.x_root > x and event.x_root < x + 32 and event.y_root > y and event.y_root < y + 32):
				self.addQueue()
		
	def redrawAddTabButton(self, w, e):
		icon = gtk.Image().render_icon(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU)
		#icon = gtk.gdk.pixbuf_new_from_file('icons/track.png')
		
		alloc = self.getAddTabButtonPos()
		self.window.draw_pixbuf(None, icon, 0, 0, alloc[0], alloc[1])

	@property
	def visibleQueue(self):
		try:
			queue = self.NoteBook.get_nth_page(self.NoteBook.get_current_page())
		except:
			queue = None

		return queue


	def addSelection(self, tracks, queue=None):
		if(queue == None):
			queue = self.visibleQueue
		queue.addTracks(tracks)
		
		
	def addPlaylist(self, label, IDList):
		db = DB()
		playlist = self.addQueue(label=label)
		self.addSelection(db.getTracksFromIDs(IDList), playlist)
		playlist.watchForChange()
		
	def addQueue(self, button=None, label=None):
		pageCount = self.NoteBook.get_n_pages()
		if(label != None):
			newQueue = Playlist(self, label)
		else:
			label = _("Queue ") + str(pageCount)
			newQueue = Queue(self, label)
			
		self.NoteBook.set_current_page(pageCount)
		return newQueue

		
	
	def cleanDirectList(self):
		"""
			Remove all DirectIter that are no longer present FIXME
		"""
		for di in self.directList:
			if(di.queue.get_path(di.ligne) == None):
				self.directList.remove(di)

	
	def closeQueue(self, button=None, tab=None):
		if(button == None):
			# Called from a shortcut
			pageNum = self.NoteBook.get_current_page()
		else:
			pageNum = self.NoteBook.page_num(tab)
		if(self.NoteBook.get_nth_page(pageNum).modified == True):
			dialog = gtk.Dialog(title=_("Closing non-saved playlist"), buttons=(gtk.STOCK_NO, gtk.RESPONSE_REJECT, gtk.STOCK_YES, gtk.RESPONSE_ACCEPT))
			box = dialog.get_content_area()
			box.pack_start(gtk.Label(_("Save changes?")), False, 5, 5)
			box.show_all()
			reponse = dialog.run()
			dialog.destroy()
			if(reponse == -3): # Accept
				self.NoteBook.get_nth_page(pageNum).save()
		self.NoteBook.remove_page(pageNum)
		# There is no queues anymore, add one
		if(self.NoteBook.get_n_pages() == 0):
			self.addQueue()
			
			
	def filter(self, entry):
		'''
			Hides visibleQueue tracks that does not match entry text
		'''
		def match(model, iter, word):
			value = model.get_value(iter, 5)
			print value
			return value.lower().find(word) != -1

		entry.get_text()
		filterModel = self.visibleQueue.model.filter_new()
		filterModel.set_visible_func(match, entry.get_text())
		self.visibleQueue.TreeView.set_model(filterModel)

		
	def getDefaultTrack(self):
		return self.visibleQueue.getTrackAt(0)
	


        @util.threaded
        def loadState(self):
		"""
			TODO Use one sql query by queue (using parametized IN clause) and thread this
			self.memcursor.execute('''SELECT id, matbefore, matafter, name, date 
                            FROM main 
                           WHERE name IN (%s)''' % 
                       ','.join('?'*len(offset)), (offset,))
		"""
		db = DB()
		queues = settings.get_option('session/queues', None)
		if(queues is not None):
			for key in queues.iterkeys():
				if type(key).__name__=='int':
					self.addQueue()
					self.addSelection(db.getTracksFromIDs(queues[key]))
				else:
					playlist = self.addPlaylist(key, queues[key])
		else:
			self.addQueue()
		

		

	def save_state(self):
		i = 0
		queues = {}
		while( i < self.NoteBook.get_n_pages()):
			t = []
			queue =  self.NoteBook.get_nth_page(i)
			model = queue.model
			iter = model.get_iter_first()
			while(iter is not None):
				t.append(model.get_value(iter, 3))
				iter = model.iter_next(iter)
			if(queue.__class__.__name__=='Playlist'):
				queues[self.NoteBook.get_nth_page(i).tab_label.get_text()] = t
			else:
				queues[i] = t
			i += 1
		settings.set_option('session/queues', queues)
		
		
	def setupShortcuts(self):
		shortcuts = (
			('<Control>W', lambda *e: self.closeQueue()),
			('<Control>T', lambda *e: self.addQueue()),
			('space', lambda *e: self.playerWidget.togglePause())
		)
		accel_group = gtk.AccelGroup()
		for key, function in shortcuts:
			key, mod = gtk.accelerator_parse(key)
			accel_group.connect_group(key, mod, gtk.ACCEL_VISIBLE,
				function)
		messager.emit('someShortcuts', self, accel_group)
		
	def scrollToCurrent(self, button=None):
		currentQueue, currentTrack = self.playerWidget.getCurrents()
		index = currentQueue.tracks.index(currentTrack)
		self.NoteBook.set_current_page(self.NoteBook.page_num(currentQueue))
		currentQueue.TreeView.scroll_to_cell(index)
			


class Queue(gtk.ScrolledWindow):
	'''
		A queue (tab) from the queue manager
	'''
	def __init__(self, manager, label):
		self.modified = False # for saved playlists
		self.manager = manager
		
		#font style, icon_playing, icon_stopping, ID, path, title, album, artist, length, playcount, rating_pixbuf, rating, bridge_src key
		self.model = gtk.ListStore(str, gtk.gdk.Pixbuf, gtk.gdk.Pixbuf, int, str, str, str, str, str, int, gtk.gdk.Pixbuf, int, str)
		self.TreeView = gtk.TreeView(self.model)
		self.TreeView.set_rules_hint(True)
		self.TreeView.set_reorderable(True)
		self.TreeView.connect("row-activated", self.activated)
		self.TreeView.connect("button-press-event", self.onButtonClick)
		self.TreeView.connect("key-press-event", self.executeShortcut)
		self.model.connect('row-deleted', self.onRowDeleted)
		self.model.connect('row-inserted', self.onRowInserted)
		self.model.connect('rows-reordered', self.onRowsReordered)
		self.isMoving = False # Tells if there is currently a drag operation initiated by this TreeView
		
		# Handle tab label here
		tab_label_box = gtk.EventBox()
		tab_label_box.set_visible_window(False) # make event box don't change anything lookwise
		tab_label_box.connect('event', self.on_tab_click, self.TreeView)
		self.tab_label = gtk.Label(label)
		self.tab_label.set_max_width_chars(20)
		hbox = gtk.HBox(False, 2)
		hbox.pack_start(self.tab_label, False, False)
		close_icon = gtk.Image()
		close_icon.set_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
		close_button = gtk.Button()
		close_button.set_relief(gtk.RELIEF_NONE)
		close_button.set_focus_on_click(False)
		close_button.set_tooltip_text(_("Close Tab"))
		close_button.add(close_icon)
		close_button.set_size_request(24,24) # avoid big padding
		close_button.connect('clicked', manager.closeQueue, self)
		hbox.pack_end(close_button, False, False)
		tab_label_box.add(hbox)
		tab_label_box.show_all()
		
		cell = gtk.CellRendererText()
		pb = gtk.CellRendererPixbuf()
		pb2 = gtk.CellRendererPixbuf()
		cellr = etoiles.RatingCellRenderer()
		cellr.connect('rating-changed', self.on_cell_rating_changed)
		#col = gtk.TreeViewColumn('header', cell, text=0, font=1)
		
		titleColumn = BSColumn('col_title', _('Title'), expand=True)
		artistColumn = BSColumn('col_artist', _('Artist'), expand=True)
		albumColumn = BSColumn('col_album', _('Album'), expand=True)
		lengthColumn = BSColumn('col_length', _('Length'))
		playcountColumn = BSColumn('col_playcount', _('Playcount'), cell, model_text=9)
		ratingColumn = BSColumn('col_rating', _('Rating'), cellr, pixbuf=10, default_width=85)
		titleColumn.name = 'col_title'
		artistColumn.name = 'col_artist'
		self.TreeView.append_column(titleColumn)
		self.TreeView.append_column(albumColumn)
		self.TreeView.append_column(artistColumn)
		self.TreeView.append_column(lengthColumn)
		self.TreeView.append_column(playcountColumn)
		self.TreeView.append_column(ratingColumn)
		
		self.TreeView.set_enable_search(False)
		self.TreeView.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
		#self.TreeView.set_rubber_banding(True) # multiple selection by dragging
		# TreeView accept dropped data
		self.TreeView.enable_model_drag_dest([('text/plain', 0, 0), ('GTK_TREE_MODEL_ROW', 0, 0)], gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_MOVE)
		self.TreeView.connect("drag-data-received", self.on_drag_data_received)
		self.TreeView.connect("drag-begin", self.on_drag_begin)
		
		titleColumn.pack_start(pb, False)
		titleColumn.pack_start(pb2, False)
		titleColumn.pack_start(cell, True)
		artistColumn.pack_start(cell, True)
		albumColumn.pack_start(cell, True)
		lengthColumn.pack_start(cell, True)
		
		titleColumn.set_attributes(cell, text=5, font=0)
		titleColumn.add_attribute(pb, 'pixbuf', 1)
		titleColumn.add_attribute(pb2, 'pixbuf', 2)
		artistColumn.add_attribute(cell, 'text', 7)
		albumColumn.add_attribute(cell, 'text', 6)
		lengthColumn.add_attribute(cell, 'text', 8)
		
		titleColumn.set_sort_column_id(5)
		albumColumn.set_sort_column_id(6)
		artistColumn.set_sort_column_id(7)
		playcountColumn.set_sort_column_id(9)
		ratingColumn.set_sort_column_id(11)

		
		self.columnsFields = {5:'title', 6:'album', 7:'artist', 9:'playcount', 11:'rating'}
		
		cols = [titleColumn, artistColumn, albumColumn, playcountColumn, ratingColumn]
		
		for col in cols:
			col.connect('clicked', self.on_column_clicked)
		
		
		#titleColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
		#titleColumn.set_fixed_width(40)
		
		messager.connect(self.updateView, 'track_data_changed')
		
		self.tracks = []
		
		
		gtk.ScrolledWindow.__init__(self)
		self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.add(self.TreeView)
		manager.NoteBook.append_page(self, tab_label_box)
		manager.show_all()
		
	def activated(self, TreeView, i , c):
		#W = TreeView, c = column, i = line number
		self.manager.playerWidget.cleanPlayFlag()
		self.manager.playerWidget.playTrack(self.tracks[i[0]], self)
		
		
	def addTracks(self, tracks, destRow=None):
		if type(tracks).__name__!='list': # One track
			tracks = [tracks,]
		
		
		
		try:
			iter_pos = self.model.get_iter(destRow[0])
			pos_type = destRow[1]
			pos = destRow[0][0]
			# Adjustment
			if pos_type == gtk.TREE_VIEW_DROP_AFTER or pos_type == gtk.TREE_VIEW_DROP_INTO_OR_AFTER:
				pos += 1
		except:
			iter_pos = None
			pos_type = None
			pos = len(self.tracks)
			
		print pos
		if(pos == len(self.tracks)):
			self.tracks.extend(tracks)
		else:
			for track in tracks:
				self.tracks.insert(pos, track)
			
		for track in tracks:
			length = self.formatLength(track.length)
			rating= IM.rating_pixbufs[track.rating]
			if(pos_type == gtk.TREE_VIEW_DROP_BEFORE or pos_type == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE):
				self.model.insert_before(iter_pos, [None, None, None, int(track.ID), track.path, track.tags['title'], track.tags['album'], track.tags['artist'], length, track.playcount, rating, track.rating, None] )
			elif(pos_type == gtk.TREE_VIEW_DROP_AFTER or pos_type == gtk.TREE_VIEW_DROP_INTO_OR_AFTER):
				self.model.insert_after(iter_pos, [None, None, None, int(track.ID), track.path, track.tags['title'], track.tags['album'], track.tags['artist'], length, track.playcount, rating, track.rating, None] )
			else:
				self.model.append([None, None, None, int(track.ID), track.path, track.tags['title'], track.tags['album'], track.tags['artist'], length, track.playcount, rating, track.rating, None] )
		
		
	def removeTrack(self, iter):
		self.model.remove(iter)
		path = self.model.get_path(iter)
		self.tracks.remove(path[0])
	

	
	
	def executeShortcut(self, widget, event):
		if(event.keyval == gtk.gdk.keyval_from_name("Delete")):
			selection = self.TreeView.get_selection()
			model, paths = selection.get_selected_rows()
			paths.reverse() # Start from tail in order to be sure next paths to delete are still referring to the same tracks as before
			for path in paths:
				model.remove(model.get_iter(path))
				self.tracks.remove(path[0])
				#next = model.iter_next(iter)
				#if(next):
					#selection.select_iter(next)
				#model.remove(iter)
			self.manager.cleanDirectList()
				
		elif(event.keyval == gtk.gdk.keyval_from_name("s")):
			selection = self.TreeView.get_selection()
			model, paths = selection.get_selected_rows()
			for path in paths:
				self.toggleStopFlag(self.tracks[path[0]])
		elif(event.keyval == gtk.gdk.keyval_from_name("space")):
			return True
		


	@staticmethod
	def formatLength(length):
		'''
			@param : seconds
			@return : standard readable time string hh:mm:ss
		'''
		minutes = 0
		while length > 59:
			length -= 60
			minutes += 1
		if length < 10:
			length = "0" + str(length)
		else:
			length = str(length)
		length = str(minutes) + ":" + length
		
		return length
		
	def getNextTrack(self, tr):
		try:
			return self.tracks[self.tracks.index(tr) + 1]
		except IndexError:
			return None
			
	def getTrackAt(self, i):
		try:
			return self.tracks[i]
		except IndexError:
			return None
			
			
	def onButtonClick(self, TreeView, event):
		# Right click
		if event.button == 3:
			from abstract.playerwidget import AbstractPlayerWidget
			path = TreeView.get_path_at_pos(int(event.x),int(event.y))[0]
			if(path != None):
				queue = TreeView.get_model()
				track_ID = queue[path][3]
				line = queue.get_iter(path)
			if (path != None):
				track = self.tracks[path[0]]
				iter = queue.get_iter(path)
				row = gtk.TreeRowReference(queue, path)
				m = TrackMenu(track_ID)
				m.append(gtk.SeparatorMenuItem())
				i = gtk.ImageMenuItem(_("Remove from queue"))
				i.set_image(gtk.image_new_from_stock(gtk.STOCK_REMOVE, gtk.ICON_SIZE_MENU))
				i.connect_object('activate', self.removeTrack, line)
				m.append(i)
				
				i = gtk.ImageMenuItem(_("Set stop cursor"))
				i.set_image(gtk.image_new_from_stock(gtk.STOCK_MEDIA_STOP, gtk.ICON_SIZE_MENU))
				i.connect_object('activate', self.toggleStopFlag, track)
				m.append(i)
				
				jumpListSize = str(len(self.manager.playerWidget.jumpList))
				i = gtk.ImageMenuItem(_("Add to perm jump list"))
				i.set_image(gtk.image_new_from_pixbuf(icons.MANAGER.pixbuf_from_text(jumpListSize, (18, 18))))
				i.connect_object('activate', AbstractPlayerWidget.addToJumpList, self.manager.playerWidget, self, track)
				m.append(i)
				
				#image = icons.MANAGER.pixbuf_from_text(str(len(self.directList)), (18, 18), '#FFCC00', '#000', '#000')
				#image = icons.MANAGER.pixbuf_from_text(str(len(self.directList)), (18, 18))
				i = gtk.ImageMenuItem(_("Add to temp jump list"))
				i.set_image(gtk.image_new_from_pixbuf(icons.MANAGER.pixbuf_from_text(jumpListSize, (18, 18), '#FFCC00', '#000', '#000')))
				i.connect_object('activate', AbstractPlayerWidget.addToJumpList, self.manager.playerWidget, self, track, True)
				m.append(i)
				
				
				# *** BRIDGES ***
				dic = self.manager.playerWidget.bridgesSrc
				
				
				if(track.bridgeSrc != None):
					def remove_bridge_src(*args):
						dic.pop(track.bridgeSrc)
						track.bridgeSrc = None
						self.refreshView(track)
						
					i = gtk.ImageMenuItem(_("Unset bridge source"))
					i.set_image(gtk.image_new_from_pixbuf(icons.MANAGER.pixbuf_from_text(track.bridgeSrc, (18, 18), '#FF0000')))
					i.connect('activate', remove_bridge_src)
				else:
					def add_bridge_src(*args):
						self.manager.playerWidget.addBridgeTrack(self, track, True)
						# gtk.TreeRowReference(self.model, path)
					
					letter = chr(65 + len(dic))
					icon = icons.MANAGER.pixbuf_from_text(letter + ' →', (24, 18), '#58FA58', '#000', '#000')
					i = gtk.ImageMenuItem(_("Add bridge source"))
					i.set_image(gtk.image_new_from_pixbuf(icon))
					i.connect('activate', add_bridge_src)
				m.append(i)
				
				# --- BRIDGES DESTINATIONS --- 
				dicDest = self.manager.playerWidget.bridgesDest
				

				if(track.bridgeDest != None):
					def remove_bridge_dest(*args):
						self.manager.playerWidget.bridgesDest.pop(track.bridgeDest)
						track.bridgeDest = None
						self.refreshView(track)
						
					i = gtk.ImageMenuItem(_("Unset bridge dest"))
					i.set_image(gtk.image_new_from_pixbuf(icons.MANAGER.pixbuf_from_text(track.bridgeDest, (18, 18), '#FF0000')))
					i.connect('activate', remove_bridge_dest)
				else:
					def add_bridge_dest(*args):
						self.manager.playerWidget.addBridgeTrack(self, track, False)#gtk.TreeRowReference(self.model, path)
					
					letterDest = chr(65 + len(dicDest))
					icon = icons.MANAGER.pixbuf_from_text('← ' + letterDest, (24, 18), '#CC2EFA')
					i = gtk.ImageMenuItem(_("Add bridge dest"))
					i.set_image(gtk.image_new_from_pixbuf(icon))
					i.connect('activate', add_bridge_dest)
					
				
					
				#if(iter in self.manager.bridges_src.values):

				#else:
					#i = gtk.ImageMenuItem(_("Add to temp jump list"))
					#i.set_image(gtk.image_new_from_pixbuf(icons.MANAGER.pixbuf_from_text('A'), (18, 18), '#FFCC00', '#000', '#000')))
					#i.connect('activate', self.manager.addToDirectList, queue, ligne, True)
				m.append(i)
				
				m.show_all()
				m.popup(None, None, None, event.button, event.time)
				
		elif event.button == 2: # Middle click is a jump flag shortcut
			path = TreeView.get_path_at_pos(int(event.x),int(event.y))[0]
			if(path != None):
				track = self.tracks[path[0]]
				from gtk.gdk import CONTROL_MASK, SHIFT_MASK
				if event.state & CONTROL_MASK:
					self.manager.playerWidget.addToJumpList(self, track, False)
				else:
					self.manager.playerWidget.addToJumpList(self, track, True)
					
					
					
	def on_cell_rating_changed(self, widget, path, rating):
		self.tracks[path[0]].change_rating(widget, rating)
	
	def on_column_clicked(self, column):
		def disable_sorting_state():
			time.sleep(2.0)
			self.model.set_sort_column_id(-2, 0)
			
		a = threading.Thread(target=disable_sorting_state)
		a.start()
		
	def on_drag_begin(self, treeview, dragcontext):
		self.isMoving = True
	
	def on_drag_data_received(self, TV, drag_context, x, y, selection_data, info, timestamp):
		if(selection_data.get_target() == 'text/plain'):
			# end of a DND
			destRow = self.TreeView.get_dest_row_at_pos(x, y)
			
			T = eval(selection_data.get_text()) # eval => turn string into dictionnary list
			db = DB()
			self.addTracks(db.getTracks(T), destRow)
		
	def on_tab_click(self, widget, event, child):
		if event.type == gtk.gdk.BUTTON_PRESS:
			if(event.button  == 3):
				m = gtk.Menu() 
				i = gtk.MenuItem(_("Save as...")) 
				i.connect('activate', self.saveAs)
				m.append(i) 
				i = gtk.MenuItem(_("Set all track to stop"))
				i.connect_object('activate', self.toggleStopFlag, self.model)
				m.append(i)
				m.show_all()
				m.popup(None, None, None, event.button, event.time)
			elif(event.button == 2):
				self.manager.closeQueue(None, self)
			else:
				return False
			

	def onRowDeleted(self, model, path):
		if(self.isMoving):
			previousPos = path[0]
			# Depending on the newPos and the previousPos we need to make adjustments
			if self.newPos < previousPos:
				previousPos -= 1
			else:
				self.newPos -= 1
			print 'previousPos = ' + str(previousPos)
			print 'newPos = ' + str(self.newPos)
			trackMoved = self.tracks.pop(previousPos)
			self.tracks.insert(self.newPos, trackMoved)
			self.isMoving = False # Done moving
		
	def onRowInserted(self, model, path, iter):
		if(self.isMoving):
			self.newPos = path[0]
			
	def onRowsReordered(self, model, path, iter, newOrder):
		column, order = self.model.get_sort_column_id()
		if(order == gtk.SORT_ASCENDING):
			reverse = False
		else:
			reverse = True
		self.tracks = sorted(self.tracks, key=attrgetter(self.columnsFields[column]), reverse=reverse)
		
	
	def refreshView(self, track):
		'''
			Method called by PlayerWidget, mainly to update flags icons
		'''
		font = 'normal'
		
		if('play' in track.flags):
			icon = gtk.gdk.pixbuf_new_from_file('icons/track.png')
			font = 'bold'
		elif 'permjump' in track.flags:
			icon = icons.MANAGER.pixbuf_from_text(str(track.priority), (18, 18))
		elif 'tempjump' in track.flags:
			icon = icons.MANAGER.pixbuf_from_text(str(track.priority), (18, 18), '#FFCC00', '#000', '#000')
		elif track.bridgeSrc != None:
			if(track.bridgeDest != None):
				icon = icons.MANAGER.pixbuf_from_text('← ' + track.bridgeDest + ' - ' + track.bridgeSrc + ' →', (36, 18), '#58FA58', '#000', '#000')
			else:
				icon = icons.MANAGER.pixbuf_from_text(track.bridgeSrc + ' →', (24, 18), '#58FA58', '#000', '#000')
		elif track.bridgeDest != None:
			icon = icons.MANAGER.pixbuf_from_text('← ' + track.bridgeDest, (24, 18), '#CC2EFA')
		else :
			icon = None
			
		if 'stop' in track.flags:
			stop_icon = gtk.ToolButton()
			stop_icon = stop_icon.render_icon(gtk.STOCK_MEDIA_STOP, gtk.ICON_SIZE_BUTTON, detail=None)
			font = 'italic'
		else:
			stop_icon = None
		
		index = self.tracks.index(track)
		iter = self.model.get_iter(index)
		self.model.set(iter, 0, font, 1, icon, 2, stop_icon, 4, track.path, 5, track.tags['title'], 6, track.tags['album'], 7, track.tags['artist'], 9, track.playcount, 10, IM.pixbuf_from_rating(track.rating))
		
	
	def saveAs(self, button):
		DN = gtk.Dialog()
		Entry = gtk.Entry()
		hbox = gtk.HBox()
		hbox.pack_start(gtk.Label(_("Name") + " : "), False, 5, 5)
		hbox.pack_end(Entry)
		DN.vbox.add(hbox)
		
		DN.add_button('OK', 1)
		DN.add_button(_("Cancel"), 0)
		DN.show_all()
		action = DN.run()
		name = Entry.get_text()
		DN.destroy()
		if(action == 1):
			self.save(name)
			messager.emit('playlistAdded', self, ['static', name])
	
	def save(self, name=None):
		if(name == None):
			name = self.tab_label.get_text()[1:]
		f = open(os.path.join(xdg.get_data_home(), 'playlists') + os.sep + name, 'w')
		for track in self.model:
			f.write(str(track[3]) + "\n")
		f.close()
	
	
	
	
	
	def toggleStopFlag(self, data):
		def process(track):
			flags = track.flags
			if 'stop' in flags:
				flags.remove('stop')
			else:
				flags.add('stop')
			self.refreshView(track)
			
		if data.__class__.__name__ == 'Track':
			process(data)
		else:
			for track in self.tracks:
				process(track)
				
	
	
	def updateView(self, track):
		"""
			A track data has just changed in the whole system. Checks if we have this track and update accordingly
			#model order :police, icon_playing, icon_stopping, ID, path, titre, album, artiste, length, count, pixbuf_note, note
		"""
		iter = self.model.get_iter_first()
		while(iter is not None):
			if(self.model.get_value(iter, 3) == int(track.ID)):
				self.model.set(iter, 4, track.path, 5, track.tags['title'], 6, track.tags['album'], 7, track.tags['artist'], 10, IM.pixbuf_from_rating(track.rating))
			iter = self.model.iter_next(iter)


class Playlist(Queue):
	def __init__(self, manager, label):
		Queue.__init__(self, manager, label)
		
	def setModified(self, model, path, i):
		if(not self.modified):
			self.modified = True
			self.tab_label.set_text('*' + self.tab_label.get_text())
			
	def watchForChange(self):
		self.model.connect("row-changed", self.setModified)

#class DirectIter(): DEPRECATED
	#'''
		#Représente une piste qui sera lue en priorité
	#'''
	#def __init__(self, row, temp=False):
		#self.row = row
		#self.temp = temp
	
	#def equals(self, ref):
		#if(self.row == None):
			#return False
		#return (ref.get_path() == self.get_path() and ref.get_model() == self.get_model())
		
	#def get_model(self):
		#return self.row.get_model()
	
	#def get_path(self):
		#return self.row.get_path()


		
	
class BSColumn(gtk.TreeViewColumn):
	'''
		Bullseye Special Column
	'''
	def __init__(self, name, label, cell=None, model_text=None, pixbuf=None, default_width=50, expand=False):
		kwargs = {}
		if(model_text is not None):
			kwargs['text'] = model_text
		if(pixbuf is not None):
			kwargs['pixbuf'] = pixbuf
		gtk.TreeViewColumn.__init__(self, label, cell, **kwargs)
		self.name = name
		width = settings.get_option('music/' + self.name + '_width', default_width)
		self.set_min_width(default_width)
		#self.set_max_width(width)
		self.set_expand(expand)
		self.set_resizable(True)
		self.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
		self.connect('notify::width', self.on_width_change)
		
	def on_width_change(self, *args):
		settings.set_option('music/' + self.name + '_width', self.get_width())


