# -*- coding: utf-8 -*-
import os
from PySide import QtGui, QtCore
from PySide.QtCore import Qt

from PySide.QtCore import QModelIndex, QAbstractItemModel
from operator import attrgetter

from common import messager, settings, xdg
from abstract.musicpanel import LibraryPanelInterface
from qt.util import treemodel
from data.elements import Track
from qt.gui import menus, modales
import threading
import time

icon_track = QtGui.QPixmap('icons/track.png')
icon_artist = QtGui.QPixmap('icons/artist.png')
icon_album = QtGui.QPixmap('icons/album.png')
icon_genre = QtGui.QPixmap('icons/genre.png')
icon_year = QtGui.QPixmap('icons/year.png')
icon_rating = QtGui.QPixmap('icons/star.png')
icons = {"title":icon_track, "artist":icon_artist, "album":icon_album, "genre":icon_genre, "rating":icon_rating, "year":icon_year}
icon_size = settings.get_option('music/panel_icon_size', 32)

class BrowserPanel(QtGui.QTabWidget):
	def __init__(self, db, queueManager):
		QtGui.QTabWidget.__init__(self)
		self.addTab(LibraryPanel(db, queueManager), _('Library'))
		self.addTab(PlaylistBrowser(db, queueManager), _('Playlists'))
		self.resize(settings.get_option('music/paned_position', 200), 500)
		
class LibraryPanel(LibraryPanelInterface, QtGui.QWidget):

	
	expandRequested = QtCore.Signal(list)
	
	def __init__(self, mdb, queueManager):
		QtGui.QWidget.__init__(self)
		self.mdb = mdb
		self.queueManager = queueManager
		self.tracks = {}
		self.mode = '("artist", "album", "title")'
		
		self.TreeView = QtGui.QTreeView()
		self.TreeView.setExpandsOnDoubleClick(False)
		self.TreeView.setSortingEnabled(True)
		self.TreeView.mouseReleaseEvent = self.mouseReleaseEvent
		
		self.model = LibraryModel()
		
		
		self.notLoading = threading.Event()
		self.notLoading.set()
		self.fillModel()
		

		self.TreeView.setModel(self.model)
		self.TreeView.activated.connect(self.enqueue)
		self.expandRequested.connect(self.expandIndexes)
		
		self.TreeView.setDragEnabled(True)
		self.TreeView.setAcceptDrops(True)
		self.TreeView.setDragDropMode(QtGui.QAbstractItemView.InternalMove)
		
		# *** TreeView visual tweaks
		self.TreeView.setColumnWidth(1, settings.get_option('music/col_label_panel_width', 170))
		header = self.TreeView.header()
		header.setStretchLastSection(False)
		header.setDefaultSectionSize(60)
		header.setResizeMode(0, QtGui.QHeaderView.ResizeToContents)
		header.setResizeMode(1, QtGui.QHeaderView.Stretch)
		header.setResizeMode(2, QtGui.QHeaderView.Fixed)
		header.setResizeMode(3, QtGui.QHeaderView.Fixed)
		header.setResizeMode(4, QtGui.QHeaderView.Fixed)
		self.TreeView.setAnimated(True)
		
		# --- Mode selector ---
		
		modesModel = QtGui.QStandardItemModel()
		modesModel.appendRow([QtGui.QStandardItem('("artist", "album", "title")'), QtGui.QStandardItem(_("Artist"))])
		modesModel.appendRow([QtGui.QStandardItem('("album", "title")'), QtGui.QStandardItem(_("Album"))])
		modesModel.appendRow([QtGui.QStandardItem('("rating", "title")'), QtGui.QStandardItem(_("Rating"))])
		modesModel.appendRow([QtGui.QStandardItem('("genre", "album", "title")'), QtGui.QStandardItem(_("Genre"))])
		modesModel.appendRow([QtGui.QStandardItem('("year", "artist", "album", "title")'), QtGui.QStandardItem(_("Year - Genre"))])
		modesModel.appendRow([QtGui.QStandardItem('("year", "genre", "artist", "album", "title")'), QtGui.QStandardItem(_("Year"))])
		modesModel.appendRow([QtGui.QStandardItem('("rating", "year", "genre", "artist", "album", "title")'), QtGui.QStandardItem(_("Rating - Year - Genre"))])


		self.modeCB = QtGui.QComboBox()
		self.modeCB.setModel(modesModel)
		self.modeCB.setModelColumn(1)
		self.modeCB.currentIndexChanged[int].connect(self.changeMode)
		
		
		refreshButton = QtGui.QPushButton(QtGui.QIcon.fromTheme('view-refresh'), _('Refresh'))
		refreshButton.clicked.connect(lambda: self.fillModel(None, True))
		modLayout = QtGui.QHBoxLayout()
		modLayout.addWidget(self.modeCB, 1)
		modLayout.addWidget(refreshButton)
		
		searchLine = QtGui.QLineEdit()
		searchLine.returnPressed.connect(lambda: self.searchFor(searchLine))
		
		layout = QtGui.QVBoxLayout()
		layout.addLayout(modLayout)
		layout.addWidget(self.TreeView)
		layout.addWidget(searchLine, 1)
		
		

		#self.setSizePolicy(QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding))
		self.setLayout(layout)

		
	def addExpandPath(self, model, node):
		self.expand_paths.append(model.createIndex(node.row(), 0, node))
		
	def append(self, model, dataTuple, parentNode=None):
		if(parentNode == None):
			parentNode = model.rootItem
		return model.append(parentNode, LibraryItem(parentNode, dataTuple))
		
	
	def beginInsertRows(self, model, index):
		model.beginInsertRows(QtCore.QModelIndex(), index, index+1)
	
	def clear(self, model):
		model.reset()
		
	def endInsertRows(self, model, expand_paths):
		model.endInsertRows()
		self.expandRequested.emit(expand_paths)
	
		
	def enqueue(self, i):
		dic = i.internalPointer().getFilter()
		
		self.queueManager.addSelection(self.mdb.getTracks(dic))
		
	#def enqueue(self, i): DEPRECATED
		#mode = eval(self.modeCB.model().item(self.modeCB.currentIndex(), 0).text()) #Ex : '("artist", "album")'
		
		#dic = {}
		#level = 0
		#params = []
		#while(i.isValid()): #On remonte la chaîne tant qu'on est pas arrivé à la racine
			#level += 1
			#params.append(i.internalPointer().label)
			#i = i.parent()
			
		#params.reverse()
		
		#i = 0
		#while(i < level):
			#dic[mode[i]] = params[i]
			#i += 1
		#self.queueManager.addSelection(self.mdb.getTracks(dic))
		
	def changeMode(self, i):
		model = self.modeCB.model()
		self.mode = model.data(model.index(i, 0))
		self.fillModel()
		
	def expandIndexes(self, indexes):
		for path in indexes:
			self.TreeView.setExpanded(path, True)
		

	


	def mouseReleaseEvent(self, e):
		if e.button() == Qt.MidButton:
			i = self.TreeView.indexAt(QtCore.QPoint(e.x(), e.y()))
			if(i.column() != 0):
				i= i.sibling(i.row(), 0)
			self.TreeView.setExpanded(i, not self.TreeView.isExpanded(i))
		else:
			QtGui.QTreeView.mouseReleaseEvent(self.TreeView, e)
			

	def resizeEvent(self, event):
		settings.set_option('music/paned_position', event.size().width())
		
		
	def searchFor(self, entry):
		text = entry.text()
		self.fillModel(word=text)
		
		


		

        
class LibraryItem(treemodel.TreeItem):
	def __init__(self, parent, data):
		treemodel.TreeItem.__init__(self, data[2], parent)
		self.icon = data[0]
		self.ID = data[1]
		self.label = data[2]
		self.playcount = data[3]
		self.rating = data[4]
		self.burn = data[5]
		self.rounded_burn = data[6]
		self.is_separator = data[7]
		self.type = data[8]
		
		self.subelements = []
		
	def __str__( self ):
		return self.label
		
	def __repr__(self):
		return self.label
		
	def getFilter(self):
		'''
			ex : {"artist":"AC/DC", "album":"Back In Black", "title":"You Shook Me All Night Long"}
		'''
		dic = {}
		item = self
		while(item.parent() != None): # Don't  eval rootItem 
			dic[item.type] = item.label
			item = item.parent()
		return dic



		
		
		
		

		
		
class LibraryModel(treemodel.TreeModel):
	def __init__(self):
		treemodel.TreeModel.__init__(self)
		self.columnsFields = ('label', 'label', 'playcount', 'rating', 'burn')


	def columnCount(self, parent):
		return 5

	def data(self, index, role):
		if not index.isValid():
			return None
		item = index.internalPointer()
		if role == Qt.DisplayRole:
			if index.column() == 1:
				return item.label
			elif index.column() == 2:
				return item.playcount
			elif index.column() == 3:
				return item.rating
			elif index.column() == 4:
				return item.rounded_burn
				
		elif role == Qt.DecorationRole and index.column() == 0:
			if(item.icon is None):
				try:
					path = os.path.join(xdg.get_thumbnail_dir(item.type + '/medium'),  item.label.replace ('/', ' ')) # + '.jpg')
					item.icon = QtGui.QPixmap(path)
					if(item.icon.isNull()):
						item.icon = icons[item.type]
					else:
						item.icon = item.icon.scaled(icon_size, icon_size, Qt.KeepAspectRatio, Qt.SmoothTransformation) #scaledToHeight(icon_size)
						
				except:
					item.icon = icons[item.label]
				
			return item.icon
		return None

	def flags(self, index):
		return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsDragEnabled
	
	def headerData(self, section, orientation, role):
		if orientation == Qt.Horizontal and role == Qt.DisplayRole:
			if section == 0:
				return 'Icon'
			elif section == 1:
				return 'Label'
			elif section == 2:
				return _('Count')
			elif section == 3:
				return _('Rating')
			elif section == 4:
				return _('Burn')
			
		return None
		
	def mimeData(self, indexes):
		'''
			What is passed during drag operations
		'''
		data = QtCore.QMimeData()
		selection = []
		for i in indexes:
			selection.append(i.internalPointer().getFilter())
		data.setData('bullseye/library.items', str(selection))
		return data
		
	def mimeTypes(self):
		'''
			FIXME Not used, didn't manage to find out how to automatically serialize items (thus overrided mimeData instead)
		'''
		return ('bullseye/library.items',)
		
	def sort(self, columnIndex, order):
		self.layoutAboutToBeChanged.emit()
		if(order == QtCore.Qt.AscendingOrder):
			reverse = False
		else:
			reverse = True
			
		def sort(elt, reverse):
			elt.childItems = sorted(elt.childItems, key=attrgetter(self.columnsFields[columnIndex]), reverse=reverse)
			for childElt in elt.childItems:
				sort(childElt, reverse)
				
		sort(self.rootItem, reverse)
		
		self.layoutChanged.emit()
		
            

class PlaylistBrowser(QtGui.QWidget):
	
	PLAYLIST_ICON_PATH = xdg.get_data_dir() + 'icons' + os.sep + 'playlist.png'
	FOLDER_ICON_PATH = xdg.get_data_dir() + 'icons' + os.sep + 'folder.png'
	FOLDER = os.path.join(xdg.get_data_home(), 'playlists')
	#PLAYLIST_ICON = QtGui.QPixmap(xdg.get_data_dir() + 'icons/playlist.png')
	#FOLDER_ICON = QtGui.QPixmap(xdg.get_data_dir() + 'icons/folder.png')
	
	def __init__(self, db, queueManager):
		self.mdb = db
		self.queueManager = queueManager
		
		QtGui.QWidget.__init__(self)
		treeView = QtGui.QTreeView()
		treeView.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
		treeView.customContextMenuRequested.connect(self.showContextMenu)
		self.model = treemodel.SimpleTreeModel()
		
		self.treeView = treeView
		treeView.setModel(self.model)
		self.reloadModel()
		
		treeView.activated.connect(self.onActivated)
		
		
		layout = QtGui.QVBoxLayout()
		layout.addWidget(treeView)
		self.setLayout(layout)
		
	def addPlaylist(self, key, text, iconPath=None, parent=None):
		if iconPath is None:
			iconPath = self.FOLDER_ICON_PATH
		node = treemodel.SimpleTreeItem(parent, key, iconPath, text)
		self.model.append(parent, node)
		return node
		
	def deletePlaylist(self, index):
		item = index.internalPointer()
		name = item.key
		if item.parent().key == '42static':
			os.remove(self.FOLDER + os.sep +  name)
		else:
			os.remove(self.FOLDER + os.sep + 'dynamic' + os.sep + name)
		#self.model.removeRow(index.row(), index.parent())
		self.reloadModel()
		
	def onActivated(self, index):
		item = index.internalPointer()
		if item.parent().key == '42static':
			f = open(self.FOLDER + os.sep +  item.key, 'r')
			IDList = f.readlines()
			f.close()
			self.queueManager.addPlaylist(item.key, IDList)
		elif item.parent().key == '42dynamic':
			f = open(self.FOLDER + os.sep + 'dynamic' + os.sep + item.key, 'r')
			data = f.readlines()
			f.close()
			self.queueManager.addQueue()
			self.queueManager.addSelection(self.mdb.getDynamicListTracks(eval(data[0])))
		
	def reloadModel(self):
		self.model.reset()
		self.staticNode = self.addPlaylist('42static', _("Static playlists"))
		for f in os.listdir(self.FOLDER):
			if os.path.isfile(os.path.join(self.FOLDER, f)):
				self.addPlaylist(f, f, self.PLAYLIST_ICON_PATH, self.staticNode)
		
		self.dynamicNode = self.addPlaylist('42dynamic', _("Dynamic playlists"))
		dossier = self.FOLDER + os.sep + 'dynamic' + os.sep
		for f in os.listdir(dossier):
			if os.path.isfile(os.path.join(dossier, f)):
				self.addPlaylist(f, f, self.PLAYLIST_ICON_PATH, self.dynamicNode)
				
		self.treeView.expandAll()
				
	def showContextMenu(self, pos):
		item = self.sender().indexAt(pos).internalPointer()
		popMenu = QtGui.QMenu(self)
		if item.key == '42static' or item.key == '42dynamic':
			addStatic = popMenu.addAction(QtGui.QIcon.fromTheme('list-add'), _("Add a static playlist"))
			addDynamic = popMenu.addAction(QtGui.QIcon.fromTheme('list-add'), _("Add a dynamic playlist"))
			
			action = popMenu.exec_(self.sender().mapToGlobal(pos))
			if action == addStatic:
				print 'todo'
			elif action == addDynamic:
				d = modales.DynamicPlaylistCreator()
				newName = d.exec_()
				if newName != None:
					self.addPlaylist(newName, newName, None, self.dynamicNode)
		else:
			edit = popMenu.addAction(QtGui.QIcon.fromTheme('list-edit'), _("Edit playlist"))
			delete = popMenu.addAction(QtGui.QIcon.fromTheme('list-remove'), _("Delete playlist"))
			
			action = popMenu.exec_(self.sender().mapToGlobal(pos))
			if action == edit:
				d = modales.DynamicPlaylistCreator(item.key)
				d.exec_()
			elif action == delete:
				self.deletePlaylist(self.sender().indexAt(pos))