# -*- coding: utf-8 -*-
import threading
import gtk
import glib

from common import messager
from data.elements import SpecialElement
from gui import menus
from gui.util import etoiles




class IconSelector(gtk.IconView):
	'''
		Widget that display elements
	'''
	def __init__(self, type):
		gtk.IconView.__init__(self)
		self.index = -1
		self.module = type

		
		self.connect("drag-begin", self.on_drag_begin)
		self.connect("drag-data-get", self.on_drag_data_get)
	
		
		#order : ID, path, label,  thumbnail, rating, categorie_ID, univers_ID
		self.model = gtk.ListStore(int, str, str, gtk.gdk.Pixbuf, int, int, int)
		self.set_model(self.model)
		#self.set_text_column(2)
		self.set_pixbuf_column(3)
		self.set_selection_mode(gtk.SELECTION_MULTIPLE)
		self.show_all()
		
		self.enable_model_drag_source(gtk.gdk.MODIFIER_MASK,  [('text/plain', 0, 0)] , gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE)
		#IconView.enable_model_drag_dest([('text/plain', 0, 0)], gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_MOVE)
		
		self.connect("item-activated", self.on_thumbnail_click)
		self.connect("button-press-event", self.on_button_press)
		
		self.openElement = None
		
		self.oneRow = True
		if(self.oneRow):
			self.set_columns(999)
			self.connect('scroll-event', self.onScroll)
			self.connect('set-scroll-adjustments', self.onSetAdj)
		
	
	def append(self, elt):	
		#self.model.append((int(elt.ID), elt.path, elt.filename, thumbnail, elt.rating, elt.c_ID, elt.u_ID))
		#glib.idle_add(self.model.append, (int(elt.ID), elt.path, elt.filename, thumbnail, elt.rating, elt.c_ID, elt.u_ID))
		glib.idle_add(self.append_element, elt)
		
	def append_element(self, elt):
		# DEPRECATED
		try:
			thumbnail = gtk.gdk.pixbuf_new_from_file(elt.thumbnail_path)
		except:
			thumbnail = gtk.gdk.pixbuf_new_from_file("icons/none.png")
		self.model.append((int(elt.ID), elt.path, elt.filename, thumbnail, elt.rating, elt.c_ID, elt.u_ID))

	
	def clear(self, b):
		self.model.clear()
	
		
	def on_button_press(self, IconView, event):
		if event.button == 1:
			self.selection = IconView.get_selected_items()
		elif event.button == 3: # Right click
			path = IconView.get_path_at_pos(int(event.x),int(event.y))
			if (path != None):
				
				tuple_ID = self.model[path][0]
				m = menus.MenuSpecialItem(self.module, tuple_ID)
				m.popup(None, None, None, event.button, event.time)
				
	def on_drag_begin(self, widget, drag_context):
		item = widget.get_selected_items()[0]
		print(item)
		print(self.selection)
		if item in self.selection:
			for item in self.selection:
				self.select_path(item)
			print('true')
		#ID = self.model[item][0]
		#selection_data.set_text(str(ID))
				
	def on_drag_data_get(self, widget, drag_context, selection_data, info, timestamp):
		items = widget.get_selected_items() # Is the index of the single clicked item (auto-selected by button-press event)
		if items[0] in self.selection: # We check that this index is not present in the previous multiple selection
			items = self.selection # If it's the case, it means that we wanted to DND the multiple selection and not the single clicked item
		T_ID = []
		for item in items:
			ID = self.model[item][0]
			T_ID.append(ID)
		selection_data.set_text(str(T_ID))
		
	def on_thumbnail_click(self, widget, i):
		print("MUST BE REIMPLEMENTED")
	
	
	def onScroll(self, widget, event):
		currentValue = self.hAdjustement.get_value()
		if event.direction == gtk.gdk.SCROLL_UP:
			step = -50#self.hAdjustement.get_page_increment()
		else:
			step = 50 #self.hAdjustement.get_page_increment()
		newVal = currentValue + step

		maxVal = self.hAdjustement.get_upper() - self.hAdjustement.get_page_size()
		if(newVal > maxVal):
			newVal = maxVal
		self.hAdjustement.set_value(newVal)
		return True
		
	def onSetAdj(self, widget, hadj, vadj):
		# Used to retrieve horizontal scroll bar adjustement upon initialisation
		self.hAdjustement = hadj
		
		

class VideoSelector(IconSelector):
	def __init__(self, player):
		IconSelector.__init__(self, "video")
		self.set_text_column(2)
		self.player = player
			
	def on_thumbnail_click(self, widget, i):
		element = SpecialElement(self.model[i][0], 'video')
		path = self.model[i][1]
		self.player.player.setUpGtkVideo(self.player.movie_window)
		self.player.player.playTrack(element)

		
		
		
class ImageSelector(IconSelector):
	"""
		TODO integrate Box_Controls
	"""
	def __init__(self, imageWidget, Box_Controls):
		IconSelector.__init__(self, "picture")
		
		self.imageWidget = imageWidget
	
		
		# --- Control ---
		B_Next = gtk.ToolButton(gtk.STOCK_MEDIA_NEXT)
		B_Prev = gtk.ToolButton(gtk.STOCK_MEDIA_PREVIOUS)
		B_Clear = gtk.ToolButton(gtk.STOCK_CLEAR)
		B_100 = gtk.ToolButton(gtk.STOCK_ZOOM_100)
		B_Zoom_Fit = gtk.ToolButton(gtk.STOCK_ZOOM_FIT)
		B_Zoom_In = gtk.ToolButton(gtk.STOCK_ZOOM_IN)
		B_Zoom_Out = gtk.ToolButton(gtk.STOCK_ZOOM_OUT)
		
		B_Prev.connect("clicked", self.loadPrevious)
		B_Next.connect("clicked", self.loadNext)
		B_Clear.connect('clicked', self.clear)
		
		B_100.connect('clicked', self.imageWidget.change_zoom, "original")
		B_Zoom_Fit.connect('clicked', self.imageWidget.change_zoom, "fit")
		B_Zoom_In.connect('clicked', self.imageWidget.change_zoom, "+")
		B_Zoom_Out.connect('clicked', self.imageWidget.change_zoom, "-")
		
		
		# DEPRECATED STUFF
		liste_categories = gtk.ListStore(int, str)
		CB = gtk.ComboBoxEntry()
		CB.set_model(liste_categories)
		CB.set_text_column(1)
		CB.set_active(0)
			
		B_Ajouter = gtk.Button(None, gtk.STOCK_ADD)
		#B_Ajouter.connect('clicked', self.set_categorie_on_image, CB)
		
		# END DEPRECATED STUFF
		
		self.rating_item = etoiles.RatingWidget(3, auto_update=False)
		self.rating_item.connect('rating-changed', self.updateRating)
		self.rating_item.set_sensitive(False)
		self.rating_item.show_all()
		
		#BB = Button Bar
		BB = gtk.HButtonBox()
		BB.add(B_Prev)
		BB.add(B_Next)
		BB.pack_end(B_Clear)
		BB.pack_start(B_Zoom_In)
		BB.pack_start(B_Zoom_Out)
		BB.pack_start(B_100)
		BB.pack_start(B_Zoom_Fit)
		Box_Controls.pack_start(BB, False)
		Box_Controls.pack_start(CB, False)
		Box_Controls.pack_start(B_Ajouter, False)
		Box_Controls.pack_start(self.rating_item, False)
		Box_Controls.show_all()
		
		self.imageWidget.set_image('icons/bullseye.svg')

		
		

		

			
			
	def displayPicture(self):
		image_path = self.model[self.index][1]
		self.imageWidget.set_image(image_path)
		self.rating_item.set_property('rating', self.model[self.index][4])
		self.rating_item.set_sensitive(True)
		#self.Image.set_size_request(img.get_width(),img.get_height())
		
		
		
	def loadNext(self, b):
		if(self.index < len(self.model) - 1):
			self.unselect_all()
			self.index += 1
			self.select_path(self.index)
			self.scroll_to_path(self.index, True, 0, 0)
			#self.IconView.item_activated(5)
			self.displayPicture()

	

	
	def updateRating(self, w, rating):
		self.model[self.index][4] = rating
		ID = self.model[self.index][0]
		self.openElement.setRating(rating)
		
	
		
	def on_thumbnail_click(self, widget, i):
		self.openElement = SpecialElement(self.model[i][0], self.module)
		self.index = i[0]
		try:
			self.displayPicture()
		except:
			#Old trick of mine to repair a bug in moveToUCStructure
			#import os
			#if not os.path.isfile(self.openElement.path):
				#(shortname, extension) = os.path.splitext(self.openElement.file)
				#self.openElement.set_path(self.openElement.folder, shortname[:-2] + extension)
				#print shortname
				#print(shortname[:-2] + extension)
			pass
		print(self.get_visible_range())
		
		
	def loadPrevious(self, b):
		if (self.index > 0):
			self.unselect_all()
			self.index += -1
			self.select_path(self.index)
			self.scroll_to_path(self.index, True, 0, 0)
			self.displayPicture()
			
	#def set_categorie_on_image(self, b, CB): DEPRECATED but might be an idea for future function
		#if(self.index == -1):
			#return False
			
		#image_ID = self.model[self.index][0]
		#liste = CB.get_model()
		#categorie = CB.get_active_text()
		#i = 0	
		#while((i < len(liste) -1) and (liste[i][1] != categorie)):
			#i += 1
			
		#if(liste[i][1] != categorie):
			#Dialog = gtk.MessageDialog(buttons=gtk.BUTTONS_YES_NO, message_format="Cette catégorie n'existe pas, voulez vous la créer?")
			#reponse = Dialog.run()
			#Dialog.destroy()
			#if(reponse == -8): #Oui
				#messager.diffuser('nouvelle_categorieI', self, categorie)
				#messager.diffuser('liste_sectionsI', self, liste)
			#elif(reponse == -9): #Non
				#return False
		
		#i= 0
		#while((i < len(liste) -1) and (liste[i][1] != categorie)):
			#i += 1
		
		##TODO
		#if(liste[i][1] == categorie):
			#categorie_ID = liste[i][0]
			#messager.diffuser('imageINcategory', self, [image_ID, categorie_ID])
		
		
		
		
	